@@ -1,6 +1,7 @@
 ;;; GNU Guix --- Functional package management for GNU
 ;;; Copyright © 2013 Andreas Enge <andreas@enge.fr>
 ;;; Copyright © 2016 Ludovic Courtès <ludo@gnu.org>
+;;; Copyright © 2019–2021 Tobias Geerinckx-Rice <me@tobias.gr>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -25,38 +26,59 @@
   #:use-module (guix utils)
   #:use-module (gnu packages)
   #:use-module (gnu packages gnupg)
+  #:use-module (gnu packages password-utils)
+  #:use-module (gnu packages pkg-config)
   #:use-module (gnu packages popt)
-  #:use-module (gnu packages python)
-  #:use-module (gnu packages linux))
+  #:use-module (gnu packages linux)
+  #:use-module (gnu packages web))
 
 (define-public cryptsetup
   (package
    (name "cryptsetup")
-   (version "1.7.5")
+   (version "2.3.7")
    (source (origin
             (method url-fetch)
             (uri (string-append "mirror://kernel.org/linux/utils/cryptsetup/v"
                                 (version-major+minor version)
-                                "/" name "-" version ".tar.xz"))
+                                "/cryptsetup-" version ".tar.xz"))
             (sha256
              (base32
-              "1gail831j826lmpdx2gsc83lp3br6wfnwh3vqwxaa1nn1lfwsc1b"))))
+              "1a97rvi6arsj8dikh1qsvixx9rizm89k155q2ypifqlqllr530v1"))))
    (build-system gnu-build-system)
-   (inputs
-    `(("libgcrypt" ,libgcrypt)
-      ("lvm2" ,lvm2)
-      ("util-linux" ,util-linux)
-      ("popt" ,popt)))
+   (arguments
+    `(#:configure-flags
+      (list
+       ;; Argon2 is always enabled, this just selects the (faster) full version.
+       "--enable-libargon2"
+       ;; The default is OpenSSL which provides better PBKDF performance.
+       "--with-crypto_backend=gcrypt"
+       ;; GRUB 2.06 supports LUKS2, but does it reliably support all set-ups…?
+       "--with-default-luks-format=LUKS1")))
    (native-inputs
-    `(("python" ,python-wrapper)))
-   (synopsis "Hard disk encryption tool")
+    (list pkg-config))
+   (inputs
+    (list argon2
+          json-c
+          libgcrypt
+          lvm2 ; device-mapper
+          popt
+          `(,util-linux "lib"))) ;libuuid
+   (synopsis "Set up transparent encryption of block devices using dm-crypt")
    (description
-    "LUKS (Linux Unified Key Setup)/Cryptsetup provides a standard on-disk
-encryption format, which does not only facilitate compatibility among
-distributions, but which also provides secure management of multiple user
-passwords.  In contrast to existing solutions, LUKS stores all setup necessary
-setup information in the partition header, enabling the users to transport
-or migrate their data seamlessly.")
+    "Cryptsetup is a utility used to conveniently set up disk encryption based
+on the @code{dm-crypt} Linux kernel module.  It is most often used to manage
+LUKS volumes but also supports plain dm-crypt volumes and loop-AES, TrueCrypt
+(including VeraCrypt extension), and BitLocker formats.
+
+@acronym{LUKS, Linux Unified Key Setup} is the standard for hard disk encryption
+with the kernel Linux.  It provides a standard on-disk-format compatible amongst
+distributions as well as secure management of multiple user passwords.  LUKS
+stores all necessary setup information in the partition header to facilitate
+data transport and migration.
+
+The package also includes the @command{veritysetup} and @command{integritysetup}
+utilities to conveniently configure the @code{dm-verity} and @code{dm-integrity}
+block integrity kernel modules.")
    (license license:gpl2)
    (home-page "https://gitlab.com/cryptsetup/cryptsetup")))
 
@@ -81,6 +103,14 @@ files).  This assumes LIBRARY uses Libtool."
      '(#:configure-flags '("--disable-shared"
                            "--enable-static-cryptsetup"
 
+                           "--disable-veritysetup"
+                           "--disable-cryptsetup-reencrypt"
+                           "--disable-integritysetup"
+
+                           ;; The default is OpenSSL which provides better PBKDF performance.
+                           "--with-crypto_backend=gcrypt"
+
+                           "--disable-blkid"
                            ;; 'libdevmapper.a' pulls in libpthread, libudev and libm.
                            "LIBS=-ludev -pthread -lm")
 
@@ -94,8 +124,7 @@ files).  This assumes LIBRARY uses Libtool."
        #:phases (modify-phases %standard-phases
                   (add-after 'install 'remove-cruft
                     (lambda* (#:key outputs #:allow-other-keys)
-                      ;; Remove everything except the 'cryptsetup' command and
-                      ;; its friend.
+                      ;; Remove everything except the 'cryptsetup' command.
                       (let ((out (assoc-ref outputs "out")))
                         (with-directory-excursion out
                           (let ((dirs (scandir "."
@@ -109,7 +138,7 @@ files).  This assumes LIBRARY uses Libtool."
                                                                     ".static")
                                                      file)
                                         (remove-store-references file))
-                                      '("sbin/cryptsetup" "sbin/veritysetup"))
+                                      '("sbin/cryptsetup"))
                             #t))))))))
     (inputs
      (let ((libgcrypt-static
@@ -117,9 +146,10 @@ files).  This assumes LIBRARY uses Libtool."
               (inherit (static-library libgcrypt))
               (propagated-inputs
                `(("libgpg-error-host" ,(static-library libgpg-error)))))))
-       `(("libgcrypt" ,libgcrypt-static)
+       `(("json-c" ,json-c-0.13)
+         ("libgcrypt" ,libgcrypt-static)
          ("lvm2" ,lvm2-static)
          ("util-linux" ,util-linux "static")
-         ("util-linux" ,util-linux)
+         ("util-linux" ,util-linux "lib")
          ("popt" ,popt))))
     (synopsis "Hard disk encryption tool (statically linked)")))
