@@ -1,7 +1,7 @@
 ;;; GNU Guix --- Functional package management for GNU
 ;;; Copyright © 2015 Tomáš Čech <sleep_walker@suse.cz>
 ;;; Copyright © 2015 Daniel Pimentel <d4n1@member.fsf.org>
-;;; Copyright © 2015, 2016, 2017, 2018, 2019, 2020 Efraim Flashner <efraim@flashner.co.il>
+;;; Copyright © 2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022 Efraim Flashner <efraim@flashner.co.il>
 ;;; Copyright © 2017 Nikita <nikita@n0.is>
 ;;; Copyright © 2018, 2019, 2020 Tobias Geerinckx-Rice <me@tobias.gr>
 ;;; Copyright © 2018 Timo Eisenmann <eisenmann@fn.de>
@@ -36,6 +36,7 @@
   #:use-module (gnu packages code)
   #:use-module (gnu packages compression)
   #:use-module (gnu packages curl)
+  #:use-module (gnu packages fonts)
   #:use-module (gnu packages fontutils)
   #:use-module (gnu packages freedesktop)
   #:use-module (gnu packages fribidi)
@@ -47,8 +48,10 @@
   #:use-module (gnu packages gnome)
   #:use-module (gnu packages gstreamer)
   #:use-module (gnu packages gtk)
+  #:use-module (gnu packages hardware)
   #:use-module (gnu packages ibus)
   #:use-module (gnu packages image)
+  #:use-module (gnu packages libunwind)
   #:use-module (gnu packages linux)
   #:use-module (gnu packages llvm)
   #:use-module (gnu packages lua)
@@ -68,7 +71,7 @@
 (define-public efl
   (package
     (name "efl")
-    (version "1.25.1")
+    (version "1.26.3")
     (source (origin
               (method url-fetch)
               (uri (string-append
@@ -76,12 +79,13 @@
                     version ".tar.xz"))
               (sha256
                (base32
-                "0svybbrvpf6q955y6fclxh3md64z0dgmh0x54x2j60503hhs071m"))))
+                "05bxc58hj0z6pkp6yy5cmy1lc575q0nrbr5lxr6z8d4kznh3my6r"))))
     (build-system meson-build-system)
     (native-inputs
      `(("check" ,check)
        ("gettext" ,gettext-minimal)
-       ("pkg-config" ,pkg-config)))
+       ("pkg-config" ,pkg-config)
+       ("python" ,python)))
     (inputs
      `(("curl" ,curl)
        ("giflib" ,giflib)
@@ -90,7 +94,7 @@
        ("ibus" ,ibus)
        ("mesa" ,mesa)
        ("libraw" ,libraw)
-       ("librsvg" ,librsvg)
+       ("librsvg" ,(librsvg-for-system))
        ("libspectre" ,libspectre)
        ("libtiff" ,libtiff)
        ("libxau" ,libxau)
@@ -126,6 +130,7 @@
        ("libjpeg" ,libjpeg-turbo)
        ("libsndfile" ,libsndfile)
        ("libpng" ,libpng)
+       ("libunwind" ,libunwind)
        ("libx11" ,libx11)
        ("libxkbcommon" ,libxkbcommon)
        ("luajit" ,luajit)
@@ -135,20 +140,18 @@
        ("wayland" ,wayland)
        ("zlib" ,zlib)))
     (arguments
-     `(#:configure-flags '("-Dembedded-lz4=false"
-                           "-Dbuild-examples=false"
-                           "-Decore-imf-loaders-disabler=scim"
-                           "-Dglib=true"
-                           "-Dmount-path=/run/setuid-programs/mount"
-                           "-Dunmount-path=/run/setuid-programs/umount"
-                           ;(string-append "-Ddictionaries-hyphen-dir="
-                           ;               (assoc-ref %build-inputs "hyphen")
-                           ;               "/share/hyphen")
-                           "-Dnetwork-backend=connman"
-                           ;; for wayland
-                           "-Dwl=true"
-                           "-Ddrm=true")
-       #:tests? #f ; Many tests fail due to timeouts and network requests.
+     `(#:configure-flags
+       `("-Dembedded-lz4=false"
+         "-Dbuild-examples=false"
+         "-Decore-imf-loaders-disabler=scim"
+         "-Dglib=true"
+         "-Dmount-path=/run/setuid-programs/mount"
+         "-Dunmount-path=/run/setuid-programs/umount"
+         "-Dnetwork-backend=connman"
+         ;; For Wayland.
+         "-Dwl=true"
+         "-Ddrm=true")
+       #:tests? #f     ; Many tests fail due to timeouts and network requests.
        #:phases
        (modify-phases %standard-phases
          ;; If we don't hardcode the location of libcurl.so and others then we
@@ -170,8 +173,7 @@
                   (string-append sndfile lib libsnd)))
                (substitute* "src/lib/elput/elput_logind.c"
                  (("libelogind.so.0" libelogind)
-                  (string-append elogind "/lib/" libelogind)))
-               #t)))
+                  (string-append elogind "/lib/" libelogind))))))
          (add-after 'unpack 'fix-install-paths
            (lambda _
              (substitute* "dbus-services/meson.build"
@@ -180,11 +182,10 @@
              (substitute* "src/tests/elementary/meson.build"
                (("dir_data") "meson.source_root(), 'test-output'"))
              (substitute* "data/eo/meson.build"
-               (("'usr', 'lib'") "'./' + dir_lib"))
-             #t))
+               (("'usr', 'lib'") "'./' + dir_lib"))))
          (add-after 'unpack 'set-home-directory
            ;; FATAL: Cannot create run dir '/homeless-shelter/.run' - errno=2
-           (lambda _ (setenv "HOME" "/tmp") #t)))))
+           (lambda _ (setenv "HOME" "/tmp"))))))
     (home-page "https://www.enlightenment.org/about-efl")
     (synopsis "Enlightenment Foundation Libraries")
     (description
@@ -198,7 +199,7 @@ removable devices or support for multimedia.")
 (define-public terminology
   (package
     (name "terminology")
-    (version "1.8.1")
+    (version "1.12.1")
     (source (origin
               (method url-fetch)
               (uri
@@ -206,44 +207,45 @@ removable devices or support for multimedia.")
                               "terminology/terminology-" version ".tar.xz"))
               (sha256
                (base32
-                "1fxqjf7g30ix4qxi6366rrax27s3maxq43z2vakwnhz4mp49m9h4"))
+                "1aasddf2343qj798b5s8qwif3lxj4pyjax6fa9sfi6if9icdkkpq"))
               (modules '((guix build utils)))
               ;; Remove the bundled fonts.
               (snippet
                '(begin
                   (delete-file-recursively "data/fonts")
                   (substitute* "data/meson.build"
-                    (("subdir\\('fonts'\\)") ""))
-                  #t))))
+                    (("subdir\\('fonts'\\)") ""))))))
     (build-system meson-build-system)
     (arguments
-     `(#:configure-flags (list "-Dtests=true"
-                               (string-append "-Dedje-cc="
-                                              (assoc-ref %build-inputs "efl")
-                                              "/bin/edje_cc"))
+     `(#:configure-flags
+       (let ((efl (assoc-ref %build-inputs "efl")))
+         (list "-Dtests=true"
+               (string-append "-Dedje-cc=" efl "/bin/edje_cc")
+               (string-append "-Deet=" efl "/bin/eet")))
        #:phases
        (modify-phases %standard-phases
          (add-after 'unpack 'set-home-directory
            ;; FATAL: Cannot create run dir '/homeless-shelter/.run' - errno=2
-           (lambda _ (setenv "HOME" "/tmp") #t))
+           (lambda _ (setenv "HOME" "/tmp")))
          (replace 'check
-           (lambda _
-             (with-directory-excursion
-               (string-append "../" ,name "-" ,version "/tests")
-               (invoke "sh" "run_tests.sh" "--verbose"
-                       "-t" "../../build/src/bin/tytest"))))
+           (lambda* (#:key tests? #:allow-other-keys)
+             (when tests?
+               (with-directory-excursion
+                 (string-append "../" ,name "-" ,version "/tests")
+                 (invoke "sh" "run_tests.sh" "--verbose"
+                         "-t" "../../build/src/bin/tytest")))))
          (add-after 'install 'remove-test-binary
            (lambda* (#:key outputs #:allow-other-keys)
              ;; This file is not meant to be installed.
              (delete-file (string-append (assoc-ref outputs "out")
-                                         "/bin/tytest"))
-             #t)))))
+                                         "/bin/tytest")))))))
     (native-inputs
-     `(("gettext" ,gettext-minimal)
-       ("perl" ,perl)
-       ("pkg-config" ,pkg-config)))
+     (list gettext-minimal
+           perl
+           pkg-config
+           python))
     (inputs
-     `(("efl" ,efl)))
+     (list efl))
     (home-page "https://www.enlightenment.org/about-terminology")
     (synopsis "Powerful terminal emulator based on EFL")
     (description
@@ -256,7 +258,7 @@ contents and more.")
 (define-public rage
   (package
     (name "rage")
-    (version "0.3.1")
+    (version "0.4.0")
     (source (origin
               (method url-fetch)
               (uri
@@ -265,18 +267,18 @@ contents and more.")
                 version ".tar.xz"))
               (sha256
                (base32
-                "04fdk23bbgvni212zrfy4ndg7vmshbsjgicrhckdvhay87pk9i75"))))
+                "03yal7ajh57x2jhmygc6msf3gzvqkpmzkqzj6dnam5sim8cq9rbw"))))
     (build-system meson-build-system)
     (arguments
      '(#:phases
        (modify-phases %standard-phases
          (add-after 'unpack 'set-home-directory
            ;; FATAL: Cannot create run dir '/homeless-shelter/.run' - errno=2
-           (lambda _ (setenv "HOME" "/tmp") #t)))))
+           (lambda _ (setenv "HOME" "/tmp"))))))
     (native-inputs
-     `(("pkg-config" ,pkg-config)))
+     (list pkg-config))
     (inputs
-     `(("efl" ,efl)))
+     (list efl))
     (home-page "https://www.enlightenment.org/about-rage")
     (synopsis "Video and audio player based on EFL")
     (description
@@ -287,7 +289,7 @@ Libraries with some extra bells and whistles.")
 (define-public enlightenment
   (package
     (name "enlightenment")
-    (version "0.24.2")
+    (version "0.25.4")
     (source (origin
               (method url-fetch)
               (uri
@@ -295,21 +297,16 @@ Libraries with some extra bells and whistles.")
                               "enlightenment/enlightenment-" version ".tar.xz"))
               (sha256
                (base32
-                "1wfz0rwwsx7c1mkswn4hc9xw1i6bsdirhxiycf7ha2vcipqy465y"))
+                "18mp4ggfy7n0yz6kzx56ali0piqa84z72vnj6649l6w2dch5vnsn"))
               (patches (search-patches "enlightenment-fix-setuid-path.patch"))))
     (build-system meson-build-system)
     (arguments
      `(#:configure-flags
-       (let ((efl (assoc-ref %build-inputs "efl")))
-         (list "-Dsystemd=false"
-               "-Dpackagekit=false"
-               "-Dwl=true"
-               (string-append "-Dedje-cc=" efl "/bin/edje_cc")
-               (string-append "-Deldbus-codegen=" efl "/bin/eldbus-codegen")
-               (string-append "-Deet=" efl "/bin/eet")))
+       (list "-Dsystemd=false"
+             "-Dpackagekit=false"
+             "-Dwl=true")
        #:phases
        (modify-phases %standard-phases
-         (delete 'bootstrap) ; We don't want to run the autogen script.
          (add-before 'configure 'set-system-actions
            (lambda* (#:key inputs #:allow-other-keys)
              (setenv "HOME" "/tmp")
@@ -317,6 +314,7 @@ Libraries with some extra bells and whistles.")
                    (setxkbmap (assoc-ref inputs "setxkbmap"))
                    (libc      (assoc-ref inputs "libc"))
                    (bc        (assoc-ref inputs "bc"))
+                   (ddcutil   (assoc-ref inputs "ddcutil"))
                    (efl       (assoc-ref inputs "efl")))
                ;; We need to patch the path to 'base.lst' to be able
                ;; to switch the keyboard layout in E.
@@ -342,13 +340,15 @@ Libraries with some extra bells and whistles.")
                                  "/run/current-system/profile/sbin")))
                (substitute* "src/modules/everything/evry_plug_calc.c"
                  (("bc -l") (string-append bc "/bin/bc -l")))
+               (substitute* "src/bin/system/e_system_ddc.c"
+                 (("libddcutil\\.so\\.?" libddcutil)
+                  (string-append ddcutil "/lib/" libddcutil)))
                (substitute* "data/etc/meson.build"
                  (("/bin/mount") "/run/setuid-programs/mount")
                  (("/bin/umount") "/run/setuid-programs/umount")
                  (("/usr/bin/eject") "/run/current-system/profile/bin/eject"))
                (substitute* "src/bin/system/e_system_power.c"
-                 (("systemctl") "loginctl"))
-               #t))))))
+                 (("systemctl") "loginctl"))))))))
     (native-inputs
      `(("gettext" ,gettext-minimal)
        ("pkg-config" ,pkg-config)))
@@ -357,19 +357,22 @@ Libraries with some extra bells and whistles.")
        ("bc" ,bc)
        ("bluez" ,bluez)
        ("dbus" ,dbus)
+       ("ddcutil" ,ddcutil)
        ("freetype" ,freetype)
+       ("libdrm" ,libdrm)
+       ("libexif" ,libexif)
        ("libxcb" ,libxcb)
        ("libxext" ,libxext)
        ("linux-pam" ,linux-pam)
-       ("puleseaudio" ,pulseaudio)
+       ("pulseaudio" ,pulseaudio)
        ("setxkbmap" ,setxkbmap)
        ("xcb-util-keysyms" ,xcb-util-keysyms)
        ("xkeyboard-config" ,xkeyboard-config)
        ("xorg-server-xwayland" ,xorg-server-xwayland)))
     (propagated-inputs
-     `(("efl" ,efl)
-       ("libxkbcommon" ,libxkbcommon)
-       ("wayland-protocols" ,wayland-protocols)))
+     (list efl libxkbcommon wayland-protocols
+           ;; Default font that applications such as IceCat require.
+           font-dejavu))
     (home-page "https://www.enlightenment.org/about-enlightenment")
     (synopsis "Lightweight desktop environment")
     (description
@@ -379,13 +382,10 @@ unload unused functionality, with support for touchscreen and suitable for
 embedded systems.")
     (license license:bsd-2)))
 
-(define-public enlightenment-wayland
-  (deprecated-package "enlightenment-wayland" enlightenment))
-
 (define-public python-efl
   (package
     (name "python-efl")
-    (version "1.25.0")
+    (version "1.26.0")
     (source
       (origin
         (method url-fetch)
@@ -393,7 +393,7 @@ embedded systems.")
                             "python/python-efl-" version ".tar.xz"))
         (sha256
          (base32
-          "0bk161xwlz4dlv56r68xwkm8snzfifaxd1j7w2wcyyk4fgvnvq4r"))
+          "0dj6f24n33hkpy0bkdclnzpxhvs8vpaxqaf7hkw0di19pjwrq25h"))
         (modules '((guix build utils)))
         ;; Remove files generated by Cython
         (snippet
@@ -404,8 +404,7 @@ embedded systems.")
                           (when (file-exists? generated-file)
                             (delete-file generated-file))))
                       (find-files "efl" "\\.pyx$"))
-            (delete-file "efl/eo/efl.eo_api.h")
-            #t))))
+            (delete-file "efl/eo/efl.eo_api.h")))))
     (build-system python-build-system)
     (arguments
      '(#:phases
@@ -418,22 +417,18 @@ embedded systems.")
           (lambda _
             (setenv "CFLAGS"
                     (string-append "-I" (assoc-ref %build-inputs "python-dbus")
-                                   "/include/dbus-1.0"))
-            #t))
+                                   "/include/dbus-1.0"))))
         (add-before 'check 'set-environment
           (lambda _
             ;; Some tests require write access to HOME.
             (setenv "HOME" "/tmp")
             ;; These tests try to connect to the internet.
             (delete-file "tests/ecore/test_09_file_download.py")
-            (delete-file "tests/ecore/test_11_con.py")
-            #t)))))
+            (delete-file "tests/ecore/test_11_con.py"))))))
     (native-inputs
-     `(("pkg-config" ,pkg-config)
-       ("python-cython" ,python-cython)))
+     (list pkg-config python-cython))
     (inputs
-     `(("efl" ,efl)
-       ("python-dbus" ,python-dbus)))
+     (list efl python-dbus python-packaging))
     (home-page "https://www.enlightenment.org/")
     (synopsis "Python bindings for EFL")
     (description
@@ -441,9 +436,6 @@ embedded systems.")
 Libraries stack (eo, evas, ecore, edje, emotion, ethumb and elementary).")
     (license license:lgpl3)))
 
-(define-public python2-efl
-  (package-with-python2 python-efl))
-
 (define-public edi
   (package
     (name "edi")
@@ -474,8 +466,7 @@ Libraries stack (eo, evas, ecore, edje, emotion, ethumb and elementary).")
        ("gettext" ,gettext-minimal)
        ("pkg-config" ,pkg-config)))
     (inputs
-     `(("clang" ,clang)
-       ("efl" ,efl)))
+     (list clang efl))
     (home-page "https://www.enlightenment.org/about-edi")
     (synopsis "Development environment for Enlightenment")
     (description "EDI is a development environment designed for and built using
@@ -486,59 +477,28 @@ and in creating applications based on the Enlightenment Foundation Library suite
                    license:gpl2          ; edi
                    license:gpl3))))      ; data/extra/examples/images/mono-runtime.png
 
-(define-public lekha
-  (package
-    (name "lekha")
-    (version "0.2.1")
-    (source (origin
-              (method url-fetch)
-              (uri (pypi-uri "Lekha" version))
-              (sha256
-               (base32
-                "0zr6i74ik58pbzrd7r9l7sawqbdv0r2c1a9927qkqzwga27x8j15"))))
-    (build-system python-build-system)
-    (arguments
-     `(#:tests? #f ; no test target
-       #:python ,python-2
-       #:phases
-       (modify-phases %standard-phases
-         (add-after 'unpack 'fix-data-location
-           (lambda _ (substitute* "setup.py"
-                       (("'/usr/")"'"))
-             #t)))))
-    (propagated-inputs
-     `(("python2-efl" ,python2-efl)
-       ("python2-pypdf2" ,python2-pypdf2)
-       ("python2-pyxdg" ,python2-pyxdg)))
-    (synopsis "Simple PDF viewer")
-    (description
-     "Simple PDF viewer based on the Enlightenment Foundation Libraries.")
-    (home-page "https://github.com/kaihu/lekha")
-    (license license:gpl3+)))
-
 (define-public ephoto
   (package
     (name "ephoto")
-    (version "1.5")
+    (version "1.6.0")
     (source
      (origin
        (method url-fetch)
        (uri (string-append "https://download.enlightenment.org/rel/"
                            "apps/ephoto/ephoto-" version ".tar.xz"))
        (sha256
-        (base32 "1q7v9abjp9jrs08xc7pqaac64yzax24dk1snjb9rciarzzh3mlzy"))))
-    (build-system gnu-build-system)
+        (base32 "1lvhcs4ba8h3z78nyycbww8mj4cscb8k200dcc3cdy8vrvrp7g1n"))))
+    (build-system meson-build-system)
     (arguments
      '(#:phases
        (modify-phases %standard-phases
          (add-after 'unpack 'set-home-directory
            ;; FATAL: Cannot create run dir '/homeless-shelter/.run' - errno=2
-           (lambda _ (setenv "HOME" "/tmp") #t)))))
+           (lambda _ (setenv "HOME" "/tmp"))))))
     (native-inputs
-     `(("check" ,check)
-       ("pkg-config" ,pkg-config)))
+     (list pkg-config))
     (inputs
-     `(("efl" ,efl)))
+     (list efl))
     (home-page "https://smhouston.us/projects/ephoto/")
     (synopsis "EFL image viewer/editor/manipulator/slideshow creator")
     (description "Ephoto is an image viewer and editor written using the
@@ -566,22 +526,27 @@ directories.
 (define-public evisum
   (package
     (name "evisum")
-    (version "0.5.6")
+    (version "0.6.0")
     (source
       (origin
         (method url-fetch)
         (uri (string-append "https://download.enlightenment.org/rel/apps/"
                             "evisum/evisum-" version ".tar.xz"))
         (sha256
-         (base32 "1l8pym7738kncvic5ga03sj9d5igigvmcxa9lbg47z2yvdjwzv97"))))
+         (base32 "1ip3rmp0hcn0pk6lv089cayx18p1b2wycgvwpnf7ghbdxg7n4q15"))))
     (build-system meson-build-system)
     (arguments
-     '(#:tests? #f))                    ; no tests
+     '(#:tests? #f                      ; no tests
+       #:phases
+       (modify-phases %standard-phases
+         (add-after 'unpack 'set-homedir
+           (lambda _
+             (setenv "HOME" (getcwd)))))))
     (native-inputs
-     `(("gettext" ,gettext-minimal)
-       ("pkg-config" ,pkg-config)))
+     (list gettext-minimal
+           pkg-config))
     (inputs
-     `(("efl" ,efl)))
+     (list efl))
     (home-page "https://www.enlightenment.org")
     (synopsis "EFL process viewer")
     (description
@@ -615,13 +580,9 @@ directories.
                   (string-append "join(\"" out "/share/epour\"")))
                #t))))))
     (native-inputs
-     `(("intltool" ,intltool)
-       ("python-distutils-extra" ,python-distutils-extra)))
+     (list intltool python-distutils-extra))
     (inputs
-     `(("libtorrent-rasterbar" ,libtorrent-rasterbar)
-       ("python-dbus" ,python-dbus)
-       ("python-efl" ,python-efl)
-       ("python-pyxdg" ,python-pyxdg)))
+     (list libtorrent-rasterbar python-dbus python-efl python-pyxdg))
     (home-page "https://www.enlightenment.org")
     (synopsis "EFL Bittorrent client")
     (description "Epour is a BitTorrent client based on the @dfn{Enlightenment
