@@ -1,6 +1,8 @@
 ;;; GNU Guix --- Functional package management for GNU
-;;; Copyright © 2020 Zhu Zihao <all_but_last@163.com>
+;;; Copyright © 2020, 2022 Zhu Zihao <all_but_last@163.com>
 ;;; Copyright © 2021 Tobias Geerinckx-Rice <me@tobias.gr>
+;;; Copyright © 2022 Dominic Martinez <dom@dominicm.dev>
+;;; Copyright © 2022 dan <i@dan.games>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -20,19 +22,21 @@
 (define-module (gnu packages fcitx5)
   #:use-module (guix packages)
   #:use-module (guix download)
+  #:use-module (guix gexp)
   #:use-module (guix git-download)
   #:use-module (guix build-system cmake)
   #:use-module (guix build-system copy)
   #:use-module ((guix licenses) #:prefix license:)
+  #:use-module (gnu packages anthy)
   #:use-module (gnu packages boost)
   #:use-module (gnu packages curl)
   #:use-module (gnu packages datastructures)
   #:use-module (gnu packages enchant)
   #:use-module (gnu packages freedesktop)
-  #:use-module (gnu packages gcc)
   #:use-module (gnu packages gettext)
   #:use-module (gnu packages glib)
   #:use-module (gnu packages gtk)
+  #:use-module (gnu packages ibus)
   #:use-module (gnu packages iso-codes)
   #:use-module (gnu packages kde-frameworks)
   #:use-module (gnu packages libevent)
@@ -69,13 +73,9 @@
            #t))))
     (build-system cmake-build-system)
     (inputs
-     `(("uthash" ,uthash)
-       ("libxcb" ,libxcb)
-       ("xcb-util" ,xcb-util)
-       ("xcb-util-keysyms" ,xcb-util-keysyms)))
+     (list uthash libxcb xcb-util xcb-util-keysyms))
     (native-inputs
-     `(("extra-cmake-modules" ,extra-cmake-modules)
-       ("pkg-config" ,pkg-config)))
+     (list extra-cmake-modules pkg-config))
     (home-page "https://github.com/fcitx/xcb-imdkit")
     (synopsis "Input method development support for XCB")
     (description "Xcb-imdkit is an implementation of xim protocol in XCB,
@@ -87,7 +87,7 @@ client.")
 (define-public fcitx5
   (package
     (name "fcitx5")
-    (version "5.0.5")
+    (version "5.0.19")
     (source
      (origin
        (method url-fetch)
@@ -95,7 +95,7 @@ client.")
              "https://download.fcitx-im.org/fcitx5/fcitx5/fcitx5-"
              version "_dict.tar.xz"))
        (sha256
-        (base32 "16j716xawbszkb3fxzpdza1a4czh0fvkysikjc9bfyvbwp72p6an"))))
+        (base32 "13zp6f8z590m34yjfzkw1jckvw1k0p8gzbkwm9lfzbrz9pzgdl1p"))))
     (build-system cmake-build-system)
     (arguments
      `(#:configure-flags
@@ -133,8 +133,7 @@ client.")
        ("xcb-util-wm" ,xcb-util-wm)
        ("xkeyboard-config" ,xkeyboard-config)))
     (native-inputs
-     `(("extra-cmake-modules" ,extra-cmake-modules)
-       ("pkg-config" ,pkg-config)))
+     (list extra-cmake-modules pkg-config))
     (native-search-paths
      (list (search-path-specification
             (variable "FCITX_ADDON_DIRS")
@@ -147,7 +146,7 @@ client.")
 (define-public fcitx5-lua
   (package
     (name "fcitx5-lua")
-    (version "5.0.2")
+    (version "5.0.10")
     (source
      (origin
        (method url-fetch)
@@ -155,15 +154,16 @@ client.")
              "https://download.fcitx-im.org/fcitx5/fcitx5-lua/fcitx5-lua-"
              version ".tar.xz"))
        (sha256
-        (base32 "0y5yc9102bz681f4wj6xqjxmfdmrshz3fhf39pa61718hkyy9lih"))))
+        (base32 "13vh6i7pap3h9jrjri3cfi7pcjwhlkw9g24ibfh0aykdhd8d7p99"))))
     (build-system cmake-build-system)
+    (arguments
+     (list
+      #:configure-flags
+      #~(list "-DUSE_DLOPEN=OFF")))
     (inputs
-     `(("fcitx5" ,fcitx5)
-       ("lua" ,lua)
-       ("gettext" ,gettext-minimal)
-       ("libpthread-stubs" ,libpthread-stubs)))
+     (list fcitx5 lua gettext-minimal libpthread-stubs))
     (native-inputs
-     `(("extra-cmake-modules" ,extra-cmake-modules)))
+     (list extra-cmake-modules))
     (home-page "https://github.com/fcitx/fcitx5-lua")
     (synopsis "Lua support for Fcitx 5")
     (description "Fcitx5-lua allows writing Fcitx5 extension in Lua.")
@@ -172,32 +172,29 @@ client.")
 (define-public libime
   (package
     (name "libime")
-    (version "1.0.6")
+    (version "1.0.14")
     (source
      (origin
        (method url-fetch)
        (uri (string-append "https://download.fcitx-im.org/fcitx5/libime/libime-"
                            version "_dict.tar.xz"))
        (sha256
-        (base32 "1s8gzpzyn16pg9gm7kw1nhl09drdndi6j4mjam14ymqsrfsdqr1r"))))
+        (base32 "0qvi9wpm4hx42h05bd4ngwzr4sv4x7i18m3ppzwc8n6q3sz5cfbh"))))
     (build-system cmake-build-system)
     (inputs
-     `(("fcitx5" ,fcitx5)
-       ("boost" ,boost)))
+     (list fcitx5 boost))
     (native-inputs
-     `(("gcc" ,gcc-9)                  ;for #include <filesystem> and ld support
-       ("extra-cmake-modules" ,extra-cmake-modules)
-       ("python" ,python)))             ;needed to run test
+     (list extra-cmake-modules python))             ;needed to run test
     (home-page "https://github.com/fcitx/libime")
-    (synopsis "Library for implementing generic input method")
-    (description "Libime is a library for implementing various input methods
+    (synopsis "Library for implementing generic input methods")
+    (description "Libime is a library for implementing various input method
 editors.")
     (license license:lgpl2.1+)))
 
 (define-public fcitx5-gtk
   (package
     (name "fcitx5-gtk")
-    (version "5.0.3")
+    (version "5.0.19")
     (source
      (origin
        (method url-fetch)
@@ -205,52 +202,117 @@ editors.")
                            "/fcitx5-gtk/fcitx5-gtk-"
                            version ".tar.xz"))
        (sha256
-        (base32 "18bwwj9a5v82c6ssc8560hd8vwykakvg6sfijwdb5jswk9mafvgj"))))
+        (base32 "007ls91jfbs3anvcfbza1pjbbsi7q5nrw7f3nzdbyngsl3nj6k37"))))
     (build-system cmake-build-system)
     (arguments
-     `(#:tests? #f                      ;No test
-       #:configure-flags
-       (list (string-append "-DGOBJECT_INTROSPECTION_GIRDIR="
-                            %output "/share/gir-1.0")
-             (string-append "-DGOBJECT_INTROSPECTION_TYPELIBDIR="
-                            %output "/lib/girepository-1.0")
-             ;; TODO: Enable it when Guix has GTK4.
-             "-DENABLE_GTK4_IM_MODULE=Off")
-       #:phases
-       (modify-phases %standard-phases
-         (add-before 'configure 'patch-install-prefix
-           (lambda* (#:key outputs #:allow-other-keys)
-             (let ((out (assoc-ref outputs "out"))
-                   (gtk2 (assoc-ref outputs "gtk2")))
-               ;; Install GTK+ 2 input method module to its own output.
-               (substitute* "gtk2/CMakeLists.txt"
-                 (("\\$\\{CMAKE_INSTALL_LIBDIR\\}")
-                  (string-append gtk2 "/lib")))))))))
+     (list
+      #:tests? #f                       ;No test
+      #:configure-flags
+      #~(list (string-append "-DGOBJECT_INTROSPECTION_GIRDIR="
+                             #$output "/share/gir-1.0")
+              (string-append "-DGOBJECT_INTROSPECTION_TYPELIBDIR="
+                             #$output "/lib/girepository-1.0")
+              "-DENABLE_GTK4_IM_MODULE=OFF")
+      #:phases
+      #~(modify-phases %standard-phases
+          (add-before 'configure 'patch-install-prefix
+            (lambda _
+              ;; Take care of different versions of GTK because this package
+              ;; provides IM module for GTK application to use input method.
+              (define (split-immodule gtk-version output)
+                (substitute* (string-append gtk-version "/CMakeLists.txt")
+                  (("\\$\\{CMAKE_INSTALL_LIBDIR\\}")
+                   (string-append output "/lib"))))
+
+              (let ((gtk2 #$output:gtk2)
+                    (gtk3 #$output:gtk3))
+                (for-each split-immodule
+                          '("gtk2" "gtk3")
+                          (list gtk2 gtk3))))))))
     (inputs
-     `(("fcitx5" ,fcitx5)
-       ("libxkbcommon" ,libxkbcommon)
-       ("gobject-introspection" ,gobject-introspection)
-       ("gtk2" ,gtk+-2)
-       ("gtk3" ,gtk+)
-       ("glib" ,glib)
-       ("libx11" ,libx11)
-       ("gettext" ,gettext-minimal)))
+     (list fcitx5
+           fmt
+           libx11
+           libxkbcommon
+           gettext-minimal
+           gobject-introspection
+           gtk+-2
+           gtk+
+           glib))
     (native-inputs
-     `(("extra-cmake-modules" ,extra-cmake-modules)
-       ("pkg-config" ,pkg-config)
-       ("glib" ,glib "bin")))           ;for glib-genmarshal
-    ;; TODO: Add "lib" output to reduce the closure size of "gtk2".
-    (outputs '("out" "gtk2"))
+     (list extra-cmake-modules pkg-config
+           `(,glib "bin")))           ;for glib-genmarshal
+    (outputs '("out" "gtk2" "gtk3"))
     (home-page "https://github.com/fcitx/fcitx5-gtk")
-    (synopsis "Glib based D-Bus client and GTK IM module for Fcitx 5")
-    (description "Fcitx5-gtk provides a Glib based D-Bus client and IM module
-for GTK+2/GTK+3 application.")
+    (synopsis "GLib-based D-Bus client and GTK IM module for Fcitx 5")
+    (description "Fcitx5-gtk provides the following functionality in the
+corresponding output:
+
+@table @code
+@item out
+GLib-based D-Bus client of Fcitx5.
+@item gtk2
+IM module for GTK+2 applications.
+@item gtk3
+IM module for GTK+3 applications.
+@end table")
     (license license:lgpl2.1+)))
 
+;; XXX: This package is separated from fcitx5-gtk for following reasons.
+;; 1. GTK4 has a lot more dependencies, some of which maybe unavailable on
+;;    platforms other than x86_64. See <https://issues.guix.gnu.org/53648>.
+;; 2. GTK4 now propagates pango@1.50, it will conflict with GTK3 and GTK2
+;;    (propagates pango@1.48) if they're all in the inputs of same package.
+;;    See <https://issues.guix.gnu.org/54261>.
+(define-public fcitx5-gtk4
+  (package
+    (inherit fcitx5-gtk)
+    (name "fcitx5-gtk4")
+    (arguments
+     (list
+      #:tests? #f                       ;No test
+      #:configure-flags
+      #~(list (string-append "-DCMAKE_CXX_FLAGS=-I"
+                             #$(this-package-input "fcitx5-gtk")
+                             "/include/Fcitx5/GClient"
+                             " -I" #$(this-package-input "glib")
+                             "/include/gio-unix-2.0")
+              "-DENABLE_GTK2_IM_MODULE=OFF"
+              "-DENABLE_GTK3_IM_MODULE=OFF")
+      #:phases
+      #~(modify-phases %standard-phases
+          (add-before 'configure 'fix-fcitxtheme-path
+            (lambda _
+              (substitute* "gtk4/gtk4inputwindow.cpp"
+                (("<gtk3/fcitxtheme.h>")
+                 "\"fcitxtheme.h\""))))
+          (add-before 'configure 'fix-gclient
+            (lambda* (#:key inputs #:allow-other-keys)
+              (define gclient
+                (search-input-file inputs "lib/libFcitx5GClient.so"))
+              ;; Force cmake search libFcitx5GClient.so in library search
+              ;; path instead of compiling again.
+              (substitute* "gtk4/CMakeLists.txt"
+                (("Fcitx5::GClient")
+                 gclient))))
+          (add-before 'build 'enter-gtk4-subdirectory
+            (lambda _
+              (chdir "gtk4")))
+          (add-after 'install 'leave-gtk4-subdirectory
+            (lambda _
+              (chdir ".."))))))
+    (inputs
+     (modify-inputs (package-inputs fcitx5-gtk)
+       (delete "gtk+")
+       (prepend fcitx5-gtk gtk)))
+    (outputs '("out"))
+    (synopsis "GTK4 IM module for Fcitx 5")
+    (description "Fcitx5-gtk4 provides IM module for GTK4 applications.")))
+
 (define-public fcitx5-qt
   (package
     (name "fcitx5-qt")
-    (version "5.0.5")
+    (version "5.0.15")
     (source
      (origin
        (method url-fetch)
@@ -258,7 +320,7 @@ for GTK+2/GTK+3 application.")
                            "/fcitx5-qt/fcitx5-qt-"
                            version ".tar.xz"))
        (sha256
-        (base32 "0vsvrfv0b21pfrz5n0v6p458vfr8k7km50h9bhjp1bnssampjfsb"))))
+        (base32 "1a21fgfsdpxxchpd5bjb7f2aj3fk7i4iawgs491974y7ad1kqx11"))))
     (build-system cmake-build-system)
     (arguments
      `(#:configure-flags
@@ -269,10 +331,10 @@ for GTK+2/GTK+3 application.")
      `(("fcitx5" ,fcitx5)
        ("libxcb" ,libxcb)
        ("libxkbcommon" ,libxkbcommon)
-       ("qtbase" ,qtbase)
+       ("qtbase" ,qtbase-5)
        ("gettext" ,gettext-minimal)))
     (native-inputs
-     `(("extra-cmake-modules" ,extra-cmake-modules)))
+     (list extra-cmake-modules))
     (home-page "https://github.com/fcitx/fcitx5-qt")
     (synopsis "Qt library and IM module for Fcitx 5")
     (description "Fcitx5-qt provides Qt library for development and IM module
@@ -282,10 +344,34 @@ for Qt based application.")
                    ;; and qt5/platforminputcontext.
                    license:bsd-3))))
 
+(define-public fcitx5-anthy
+  (package
+    (name "fcitx5-anthy")
+    (version "5.0.12")
+    (source
+     (origin
+       (method url-fetch)
+       (uri (string-append "https://download.fcitx-im.org/fcitx5"
+                           "/fcitx5-anthy/fcitx5-anthy-"
+                           version ".tar.xz"))
+       (sha256
+        (base32 "090wdxsd0clynw5k3l522mcrywqv5p194kjl79g8jbwa637642w0"))))
+    (build-system cmake-build-system)
+    (arguments
+     `(#:tests? #f)) ;; no tests
+    (inputs (list fcitx5 anthy gettext-minimal fmt))
+    (native-inputs
+     (list extra-cmake-modules pkg-config))
+    (home-page "https://github.com/fcitx/fcitx5-anthy")
+    (synopsis "Anthy Japanese language input for Fcitx 5")
+    (description "Fcitx5-Anthy provides Japanese input support to Fcitx5 using
+the Anthy input method.")
+    (license license:gpl2+)))
+
 (define-public fcitx5-chinese-addons
   (package
     (name "fcitx5-chinese-addons")
-    (version "5.0.3")
+    (version "5.0.15")
     (source
      (origin
        (method url-fetch)
@@ -293,10 +379,12 @@ for Qt based application.")
                            "/fcitx5-chinese-addons/fcitx5-chinese-addons-"
                            version "_dict.tar.xz"))
        (sha256
-        (base32 "1kmzbllk0g86m2z3piwn9j84ihxixyxzv7rzj832xfvwqgk7gixk"))))
+        (base32 "0l3qa958ngb19ydm4jsm44yvw2gn3p6gfjvknm4iv4fdazcr4plp"))))
     (build-system cmake-build-system)
     (arguments
-     `(#:phases
+     `(#:configure-flags
+       '("-DUSE_WEBKIT=off")
+       #:phases
        (modify-phases %standard-phases
          (add-before 'configure 'split-outputs
            ;; Build with GUI supports requires Qt and increase package closure
@@ -315,12 +403,13 @@ for Qt based application.")
        ("fmt" ,fmt)
        ("libpthread-stubs" ,libpthread-stubs)
        ("opencc" ,opencc)
-       ("qtbase" ,qtbase)
+       ("qtbase" ,qtbase-5)
        ("fcitx5-qt" ,fcitx5-qt)
-       ("qtwebkit" ,qtwebkit)))
+       ("qtdeclarative-5" ,qtdeclarative-5)
+       ("qtwebchannel-5" ,qtwebchannel-5)
+       ("qtwebengine-5" ,qtwebengine-5)))
     (native-inputs
-     `(("extra-cmake-modules" ,extra-cmake-modules)
-       ("pkg-config" ,pkg-config)))
+     (list extra-cmake-modules pkg-config))
     (outputs '("out" "gui"))
     (home-page "https://github.com/fcitx/fcitx5-chinese-addons")
     (synopsis "Chinese related addons for Fcitx 5")
@@ -346,7 +435,7 @@ including input methods previous bundled inside Fcitx 4:
 (define-public fcitx5-configtool
   (package
     (name "fcitx5-configtool")
-    (version "5.0.2")
+    (version "5.0.15")
     (source
      (origin
        (method url-fetch)
@@ -354,7 +443,7 @@ including input methods previous bundled inside Fcitx 4:
              "https://download.fcitx-im.org/fcitx5"
              "/fcitx5-configtool/fcitx5-configtool-" version ".tar.xz"))
        (sha256
-        (base32 "0rpk3yn572pcr29jak63x84g0qgj8hj011aw8gmxjah9311nxhpb"))))
+        (base32 "1k5qmwp2v8k2r1lvagdy6ffsafcqkx4k803j9a0fm5pp5yz0ibk8"))))
     (build-system cmake-build-system)
     (arguments
      `(#:configure-flags
@@ -363,7 +452,7 @@ including input methods previous bundled inside Fcitx 4:
     (inputs
      `(("fcitx5" ,fcitx5)
        ("fcitx5-qt" ,fcitx5-qt)
-       ("qtbase" ,qtbase)
+       ("qtbase" ,qtbase-5)
        ("qtx11extras" ,qtx11extras)
        ("kitemviews" ,kitemviews)
        ("kwidgetsaddons" ,kwidgetsaddons)
@@ -373,9 +462,7 @@ including input methods previous bundled inside Fcitx 4:
        ("gettext" ,gettext-minimal)
        ("iso-codes" ,iso-codes)))
     (native-inputs
-     `(("gcc" ,gcc-9)
-       ("extra-cmake-modules" ,extra-cmake-modules)
-       ("pkg-config" ,pkg-config)))
+     (list extra-cmake-modules pkg-config))
     (home-page "https://github.com/fcitx/fcitx5-configtool")
     (synopsis "Graphical configuration tool for Fcitx 5")
     (description "Fcitx5-configtool is a graphical configuration tool
@@ -385,7 +472,7 @@ to manage different input methods in Fcitx 5.")
 (define-public fcitx5-material-color-theme
   (package
     (name "fcitx5-material-color-theme")
-    (version "0.1")
+    (version "0.2.1")
     (source
      (origin
        (method git-fetch)
@@ -394,7 +481,7 @@ to manage different input methods in Fcitx 5.")
              (commit version)))
        (file-name (git-file-name name version))
        (sha256
-        (base32 "1mgc722521jmfx0xc3ibmiycd3q2w7xg2956xcpc07kz90gcdjaa"))))
+        (base32 "0drdypjf1njl7flkb5d581vchwlp4gaqyws3cp0v874wkwh4gllb"))))
     (build-system copy-build-system)
     (arguments
      `(#:phases
@@ -410,7 +497,6 @@ to manage different input methods in Fcitx 5.")
 
                (define (install-theme-variant variant target)
                  (let ((dir (string-append themes-prefix "/" target))
-                       (png (string-append "panel-" variant ".png"))
                        (conf (string-append "theme-" variant ".conf")))
                    (format #t "install: Installing color variant \"~a\" to ~a~%"
                            variant dir)
@@ -418,13 +504,15 @@ to manage different input methods in Fcitx 5.")
                      (("^Name=.*")
                       (string-append "Name=" target "\n")))
                    (mkdir-p dir)
-                   (install-file png dir)
                    (copy-file conf (string-append dir "/theme.conf"))
                    (symlink (string-append assets-dir "/arrow.png")
-                            (string-append dir "/arrow.png"))))
+                            (string-append dir "/arrow.png"))
+                   (symlink (string-append assets-dir "/radio.png")
+                            (string-append dir "/radio.png"))))
 
                (mkdir-p assets-dir)
                (install-file "arrow.png" assets-dir)
+               (install-file "radio.png" assets-dir)
                (for-each
                 (lambda (x)
                   (install-theme-variant
@@ -433,7 +521,9 @@ to manage different input methods in Fcitx 5.")
                   "orange" "pink" "red" "teal"))
 
                (install-theme-variant
-                "deepPurple" "Material-Color-DeepPurple")))))))
+                "deepPurple" "Material-Color-DeepPurple")
+               (install-theme-variant
+                "sakuraPink" "Material-Color-SakuraPink")))))))
     (home-page "https://github.com/hosxy/Fcitx5-Material-Color")
     (synopsis "Material Design for Fcitx 5")
     (description "Fcitx5-material-color-theme is a Material Design theme
@@ -451,3 +541,42 @@ for Fcitx 5 with following color variants:
 @item DeepPurple
 @end itemize\n")
     (license license:asl2.0)))
+
+(define-public fcitx5-rime
+  (package
+    (name "fcitx5-rime")
+    (version "5.0.14")
+    (source (origin
+              (method url-fetch)
+              (uri (string-append "https://download.fcitx-im.org/fcitx5"
+                                  "/fcitx5-rime/fcitx5-rime-" version
+                                  ".tar.xz"))
+              (sha256
+               (base32
+                "1i8wb7pbjifz48j4c18lp07ms1a2wc3p9d9g13dgimgf9yi64brm"))))
+    (build-system cmake-build-system)
+    (arguments
+     '(#:tests? #f ;no tests
+       #:configure-flags (list (string-append "-DRIME_DATA_DIR="
+                                              (assoc-ref %build-inputs
+                                                         "rime-data")
+                                              "/share/rime-data"))
+       #:phases (modify-phases %standard-phases
+                  (add-after 'unpack 'patch-source
+                    (lambda _
+                      (substitute* "data/CMakeLists.txt"
+                        (("DESTINATION....RIME_DATA_DIR..")
+                         "DESTINATION \"${CMAKE_INSTALL_DATADIR}/rime-data\""))
+                      #t)))))
+    (inputs (list fcitx5 librime rime-data))
+    (native-inputs (list gettext-minimal extra-cmake-modules pkg-config))
+    (home-page "https://github.com/fcitx/fcitx5-rime")
+    (synopsis "Rime Input Method Engine for Fcitx 5")
+    (description
+     "@dfn{fcitx5-rime} provides the Rime input method engine for fcitx5.
+Rime is a lightweight, extensible input method engine supporting various input
+schemas including glyph-based input methods, romanization-based input methods
+as well as those for Chinese dialects.  It has the ability to compose phrases
+and sentences intelligently and provide very accurate traditional Chinese
+output.")
+    (license license:lgpl2.1+)))
