@@ -2,11 +2,14 @@
 ;;; Copyright © 2014 Mark H Weaver <mhw@netris.org>
 ;;; Copyright © 2014 Manolis Fragkiskos Ragkousis <manolis837@gmail.com>
 ;;; Copyright © 2016 Hartmut Goebel <h.goebel@crazy-compilers.com>
-;;; Copyright © 2016, 2018 Ludovic Courtès <ludo@gnu.org>
+;;; Copyright © 2016, 2018, 2021 Ludovic Courtès <ludo@gnu.org>
 ;;; Copyright © 2016, 2019 Efraim Flashner <efraim@flashner.co.il>
 ;;; Copyright © 2017 Jonathan Brielmaier <jonathan.brielmaier@web.de>
 ;;; Copyright © 2017 Julien Lepiller <julien@lepiller.eu>
-;;; Copyright © 2018, 2019, 2020 Tobias Geerinckx-Rice <me@tobias.gr>
+;;; Copyright © 2018–2021 Tobias Geerinckx-Rice <me@tobias.gr>
+;;; Copyright © 2021 Ricardo Wurmus <rekado@elephly.net>
+;;; Copyright © 2021 Mathieu Othacehe <othacehe@gnu.org>
+;;; Copyright © 2022 Peter Polidoro <peter@polidoro.io>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -28,22 +31,29 @@
   #:use-module (guix download)
   #:use-module (guix git-download)
   #:use-module (guix packages)
+  #:use-module (guix utils)
   #:use-module (gnu packages)
   #:use-module (guix build-system cmake)
   #:use-module (guix build-system gnu)
   #:use-module (guix build-system python)
+  #:use-module (gnu packages autotools)
+  #:use-module (gnu packages admin)
+  #:use-module (gnu packages base)
   #:use-module (gnu packages bison)
+  #:use-module (gnu packages boost)
   #:use-module (gnu packages compression)
-  #:use-module (gnu packages flex)
   #:use-module (gnu packages elf)
+  #:use-module (gnu packages flex)
+  #:use-module (gnu packages ghostscript)
+  #:use-module (gnu packages gnupg)
+  #:use-module (gnu packages groff)
   #:use-module (gnu packages pciutils)
   #:use-module (gnu packages pkg-config)
   #:use-module (gnu packages libusb)
   #:use-module (gnu packages libftdi)
   #:use-module (gnu packages pciutils)
   #:use-module (gnu packages qt)
-  #:use-module (gnu packages autotools)
-  #:use-module (gnu packages admin))
+  #:use-module (gnu packages tls))
 
 (define-public flashrom
   (package
@@ -58,11 +68,8 @@
                (base32
                 "0ax4kqnh7kd3z120ypgp73qy1knz47l6qxsqzrfkd97mh5cdky71"))))
     (build-system gnu-build-system)
-    (inputs `(("dmidecode" ,dmidecode)
-              ("pciutils" ,pciutils)
-              ("libusb" ,libusb)
-              ("libftdi" ,libftdi)))
-    (native-inputs `(("pkg-config" ,pkg-config)))
+    (inputs (list dmidecode pciutils libusb libftdi))
+    (native-inputs (list pkg-config))
     (arguments
      '(#:make-flags
        (list "CC=gcc"
@@ -77,9 +84,15 @@
              (substitute* "dmi.c"
                (("\"dmidecode\"")
                 (format #f "~S"
-                        (string-append (assoc-ref inputs "dmidecode")
-                                       "/sbin/dmidecode"))))
-             #t)))))
+                        (search-input-file inputs "/sbin/dmidecode"))))))
+         (add-before 'build 'patch-type-error
+           (lambda _
+             ;; See https://github.com/flashrom/flashrom/pull/133
+             (substitute* "libflashrom.c"
+               (("supported_boards\\[i\\].working = binfo\\[i\\].working")
+                "supported_boards[i].working = (enum flashrom_test_state)binfo[i].working")
+               (("supported_chipsets\\[i\\].status = chipset\\[i\\].status")
+                "supported_chipsets[i].status = (enum flashrom_test_state)chipset[i].status")))))))
     (home-page "https://flashrom.org/")
     (synopsis "Identify, read, write, erase, and verify ROM/flash chips")
     (description
@@ -93,25 +106,28 @@ programmer devices.")
 (define-public 0xffff
   (package
     (name "0xffff")
-    (version "0.8")
+    (version "0.9")
     (source
      (origin
        (method git-fetch)
        (uri (git-reference
-             (url "https://github.com/pali/0xffff.git")
+             (url "https://github.com/pali/0xffff")
              (commit version)))
        (file-name (git-file-name name version))
        (sha256
-        (base32 "1a5b7k96vzirb0m8lqp7ldn77ppz4ngf56wslhsj2c6flcyvns4v"))))
+        (base32 "0rl1xzbxl991pm2is98zbryac1lgjrc3zphmbd8agv07av0r6r6n"))))
     (build-system gnu-build-system)
     (inputs
-     `(("libusb" ,libusb-0.1)))         ; doesn't work with libusb-compat
+     ;; Building with libusb-compat will succeed but the result will be broken.
+     ;; See <https://github.com/pali/0xFFFF/issues/3>.
+     (list libusb-0.1))
     (arguments
-     '(#:phases
+     `(#:phases
        (modify-phases %standard-phases
          (delete 'configure))           ; no configure
        #:make-flags
-       (list "CC=gcc"
+       (list (string-append "CC=" ,(cc-for-target))
+             "HOST_CC=gcc"
              "BUILD_DATE=GNU Guix"
              (string-append "PREFIX=" %output))
        #:tests? #f))                    ; no 'check' target
@@ -137,12 +153,9 @@ brick your device.")
        (base32 "15m1w1qad3dj7r8n5ng1qqcaiyx1gyd6hnc3p2apgjllccdp77qg"))))
     (build-system gnu-build-system)
     (inputs
-     `(("libelf" ,libelf)
-       ("libusb" ,libusb-compat)
-       ("libftdi" ,libftdi)))
+     (list libelf libusb-compat libftdi))
     (native-inputs
-     `(("bison" ,bison)
-       ("flex" ,flex)))
+     (list bison flex))
     (home-page "https://www.nongnu.org/avrdude/")
     (synopsis "AVR downloader and uploader")
     (description
@@ -166,9 +179,9 @@ programming} technique.")
       (patches (search-patches "dfu-programmer-fix-libusb.patch"))))
     (build-system gnu-build-system)
     (native-inputs
-     `(("pkg-config" ,pkg-config)))
+     (list pkg-config))
     (inputs
-     `(("libusb" ,libusb)))
+     (list libusb))
     (home-page "https://dfu-programmer.github.io/")
     (synopsis "Device firmware update programmer for Atmel chips")
     (description
@@ -180,7 +193,7 @@ ISP.")
 (define-public dfu-util
   (package
     (name "dfu-util")
-    (version "0.9")
+    (version "0.11")
     (source (origin
               (method url-fetch)
               (uri (string-append
@@ -188,12 +201,12 @@ ISP.")
                     version ".tar.gz"))
               (sha256
                (base32
-                "0czq73m92ngf30asdzrfkzraag95hlrr74imbanqq25kdim8qhin"))))
+                "17piiyp08pccqmbhnswv957lkypmmm92kps79hypxvw23ai3pddl"))))
     (build-system gnu-build-system)
     (inputs
-     `(("libusb" ,libusb)))
+     (list libusb))
     (native-inputs
-     `(("pkg-config" ,pkg-config)))
+     (list pkg-config))
     (synopsis "Host side of the USB Device Firmware Upgrade (DFU) protocol")
     (description
      "The DFU (Universal Serial Bus Device Firmware Upgrade) protocol is
@@ -205,56 +218,48 @@ firmware from it.")
     (license license:gpl2+)))
 
 (define-public teensy-loader-cli
-  ;; The repo does not tag versions nor does it use releases, but a commit
-  ;; message says "Importing 2.1", while the sourcce still says "2.0". So pin
-  ;; to a fixed commit.
-  (let ((commit "f289b7a2e5627464044249f0e5742830e052e360"))
-    (package
-      (name "teensy-loader-cli")
-      (version (git-version "2.1" "1" commit))
-      (source
-       (origin
-         (method git-fetch)
-         (uri (git-reference
-                (url "https://github.com/PaulStoffregen/teensy_loader_cli.git")
-                (commit commit)))
-         (sha256 (base32 "0sssim56pwsxp5cp5dlf6mi9h5fx2592m6j1g7abnm0s09b0lpdx"))
-         (file-name (git-file-name name version))
-         (modules '((guix build utils)))
-         (snippet
-          `(begin
-             ;; Remove example flash files and teensy rebooter flash binaries.
-             (for-each delete-file (find-files "." "\\.(elf|hex)$"))
-             ;; Fix the version
-             (substitute* "teensy_loader_cli.c"
-               (("Teensy Loader, Command Line, Version 2.0\\\\n")
-                (string-append "Teensy Loader, Command Line, " ,version "\\n")))
-             #t))
+  (package
+    (name "teensy-loader-cli")
+    (version "2.2")
+    (source
+     (origin
+       (method git-fetch)
+       (uri (git-reference
+             (url "https://github.com/PaulStoffregen/teensy_loader_cli")
+             (commit version)))
+       (sha256 (base32 "12n8ifz4gph1anhwd8if3j1kw0wc3yxf48abbyxl8071l9vj3m0b"))
+       (file-name (git-file-name name version))
+       (modules '((guix build utils)))
+       (snippet
+        `(begin
+           ;; Remove example flash files and teensy rebooter flash binaries.
+           (for-each delete-file (find-files "." "\\.(elf|hex)$"))
+           #t))
        (patches (search-patches "teensy-loader-cli-help.patch"))))
-      (build-system gnu-build-system)
-      (arguments
-       '(#:tests? #f ;; Makefile has no test target
-         #:make-flags (list "CC=gcc" (string-append "PREFIX=" %output))
-         #:phases
-         (modify-phases %standard-phases
-           (delete 'configure)
-           (replace 'install
-             (lambda* (#:key outputs #:allow-other-keys)
-               (let* ((out (assoc-ref outputs "out"))
-                      (bin (string-append out "/bin")))
-                 (install-file "teensy_loader_cli" bin)
-                 #t))))))
-      (inputs
-       `(("libusb-compat" ,libusb-compat)))
-      (synopsis "Command line firmware uploader for Teensy development boards")
-      (description
-       "The Teensy loader program communicates with your Teensy board when the
+    (build-system gnu-build-system)
+    (arguments
+     '(#:tests? #f ;; Makefile has no test target
+       #:make-flags (list "CC=gcc" (string-append "PREFIX=" %output))
+       #:phases
+       (modify-phases %standard-phases
+         (delete 'configure)
+         (replace 'install
+           (lambda* (#:key outputs #:allow-other-keys)
+             (let* ((out (assoc-ref outputs "out"))
+                    (bin (string-append out "/bin")))
+               (install-file "teensy_loader_cli" bin)
+               #t))))))
+    (inputs
+     (list libusb-compat))
+    (synopsis "Command line firmware uploader for Teensy development boards")
+    (description
+     "The Teensy loader program communicates with your Teensy board when the
 HalfKay bootloader is running, so you can upload new programs and run them.
 
 You need to add the udev rules to make the Teensy update available for
 non-root users.")
-      (home-page "https://www.pjrc.com/teensy/loader_cli.html")
-      (license license:gpl3))))
+    (home-page "https://www.pjrc.com/teensy/loader_cli.html")
+    (license license:gpl3)))
 
 (define-public rkflashtool
   (let ((commit "8966c4e277de8148290554aaaa4146a3a84a3c53")
@@ -266,7 +271,7 @@ non-root users.")
         (origin
           (method git-fetch)
           (uri (git-reference
-                (url "https://github.com/linux-rockchip/rkflashtool.git")
+                (url "https://github.com/linux-rockchip/rkflashtool")
                 (commit commit)))
           (file-name (git-file-name name version))
           (sha256
@@ -280,9 +285,9 @@ non-root users.")
          #:make-flags (list (string-append "PREFIX=" %output))
          #:tests? #f)) ; no tests
       (native-inputs
-       `(("pkg-config" ,pkg-config)))
+       (list pkg-config))
       (inputs
-       `(("libusb" ,libusb)))
+       (list libusb))
       (home-page "https://github.com/linux-rockchip/rkflashtool")
       (synopsis "Tools for flashing Rockchip devices")
       (description "Allows flashing of Rockchip based embedded linux devices.
@@ -326,9 +331,7 @@ RK3036, RK3066, RK312X, RK3168, RK3188, RK3288, RK3368.")
                (install-file "libpit/libpit.a" lib)
                #t))))))
     (inputs
-     `(("libusb" ,libusb)
-       ("qtbase" ,qtbase)
-       ("zlib" ,zlib)))
+     (list libusb qtbase-5 zlib))
     (home-page "https://glassechidna.com.au/heimdall/")
     (synopsis "Flash firmware onto Samsung mobile devices")
     (description "@command{heimdall} is a tool suite used to flash firmware (aka
@@ -345,7 +348,7 @@ referred to as the \"Odin 3 protocol\".")
     (source (origin
               (method git-fetch)
               (uri (git-reference
-                    (url "https://github.com/coreboot/coreboot.git")
+                    (url "https://github.com/coreboot/coreboot")
                     (commit version)))
               (file-name (git-file-name name version))
               (sha256
@@ -386,8 +389,7 @@ dump Intel Firmware Descriptor data of an image file.")
                 "0nw555i0fm5kljha9h47bk70ykbwv8ddfk6qhz6kfqb79vzhy4h2"))))
     (build-system gnu-build-system)
     (inputs
-     `(("pciutils" ,pciutils)
-       ("zlib" ,zlib)))
+     (list pciutils zlib))
     (arguments
      `(#:make-flags
        (list "CC=gcc"
@@ -419,7 +421,7 @@ Management Engine (ME).  You need to @code{sudo rmmod mei_me} and
     (source (origin
               (method git-fetch)
               (uri (git-reference
-                     (url "https://github.com/corna/me_cleaner.git")
+                     (url "https://github.com/corna/me_cleaner")
                      (commit (string-append "v" version))))
               (sha256
                (base32
@@ -447,15 +449,15 @@ ME as far as possible (it only edits ME firmware image files).")
 (define-public uefitool
   (package
     (name "uefitool")
-    (version "0.27.0")
+    (version "0.28.0")
     (source (origin
               (method git-fetch)
               (uri (git-reference
-                     (url "https://github.com/LongSoft/UEFITool.git")
+                     (url "https://github.com/LongSoft/UEFITool")
                      (commit version)))
               (sha256
                (base32
-                "1i1p823qld927p4f1wcphqcnivb9mq7fi5xmzibxc3g9zzgnyc2h"))
+                "1n2hd2dysi5bv2iyq40phh1jxc48gdwzs414vfbxvcharcwapnja"))
               (file-name (git-file-name name version))))
     (build-system gnu-build-system)
     (arguments
@@ -467,12 +469,93 @@ ME as far as possible (it only edits ME firmware image files).")
          (replace 'install
            (lambda* (#:key outputs #:allow-other-keys)
              (install-file "UEFITool" (string-append (assoc-ref outputs "out")
-                                                     "/bin"))
-             #t)))))
+                                                     "/bin")))))))
     (inputs
-     `(("qtbase" ,qtbase)))
+     (list qtbase-5))
     (home-page "https://github.com/LongSoft/UEFITool/")
     (synopsis "UEFI image editor")
     (description "@code{uefitool} is a graphical image file editor for
 Unifinished Extensible Firmware Interface (UEFI) images.")
     (license license:bsd-2)))
+
+(define-public srecord
+  (package
+    (name "srecord")
+    (version "1.64")
+    (source
+     (origin
+       (method url-fetch)
+       (uri (string-append "mirror://sourceforge/srecord/srecord/"
+                           version "/srecord-" version ".tar.gz"))
+       (sha256
+        (base32
+         "1qk75q0k5vzmm3932q9hqz2gp8n9rrdfjacsswxc02656f3l3929"))))
+    (build-system gnu-build-system)
+    (arguments
+     `(#:configure-flags
+       (list (string-append "SH="
+                            (assoc-ref %build-inputs "bash")
+                            "/bin/bash"))))
+    (inputs
+     (list boost libgcrypt))
+    (native-inputs
+     (list bison
+           diffutils
+           ghostscript
+           groff
+           libtool
+           which))
+    (home-page "http://srecord.sourceforge.net/")
+    (synopsis "Tools for EPROM files")
+    (description "The SRecord package is a collection of powerful tools for
+manipulating EPROM load files.  It reads and writes numerous EPROM file
+formats, and can perform many different manipulations.")
+    (license license:gpl3+)))
+
+(define-public uuu
+  (package
+    (name "uuu")
+    (version "1.4.165")
+    (source
+     (origin
+       (method git-fetch)
+       (uri (git-reference
+             (url "https://github.com/NXPmicro/mfgtools")
+             (commit (string-append "uuu_" version))))
+       (file-name (git-file-name name version))
+       (sha256
+        (base32
+         "0k309lp27d4k6x4qq0badbk8i47xsc6f3fffz73650iyfs4hcniw"))))
+    (arguments
+     `(#:tests? #f                      ; no tests
+       #:modules ((guix build utils)
+                  (ice-9 popen)
+                  (srfi srfi-26)
+                  (guix build cmake-build-system))
+       #:phases
+       (modify-phases %standard-phases
+         (add-before 'configure 'fix-version-gen
+           (lambda _
+             (call-with-output-file ".tarball-version"
+               (lambda (port)
+                 (display ,version port)))))
+         (add-after 'install 'install-udev-rules
+           (lambda* (#:key outputs #:allow-other-keys)
+             (let* ((out (assoc-ref outputs "out"))
+                    (uuu (string-append out "/bin/uuu"))
+                    (pipe (open-pipe* OPEN_READ uuu "-udev"))
+                    (rules
+                     (string-append out "/lib/udev/rules.d/70-uuu.rules")))
+               (mkdir-p (string-append out "/lib/udev/rules.d"))
+               (call-with-output-file rules
+                 (cut dump-port pipe <>))))))))
+    (build-system cmake-build-system)
+    (native-inputs
+     (list pkg-config))
+    (inputs
+     (list libusb bzip2 zlib libzip openssl))
+    (home-page "https://github.com/NXPmicro/mfgtools")
+    (synopsis "Freescale/NXP I.MX chip image deploy tools")
+    (description "@code{uuu} is a command line tool, evolved out of MFGTools.
+It can be used to upload images to I.MX SoC's using at least their boot ROM.")
+    (license license:bsd-3)))
