@@ -64,7 +64,7 @@
              ("libxpm" ,libxpm)
              ("zlib" ,zlib)))
    (native-inputs
-     `(("pkg-config" ,pkg-config)))
+     (list pkg-config))
    (synopsis "PostScript and PDF viewer using Ghostscript as a back-end")
    (description
     "GNU GV is a graphical user interface to the Ghostscript interpreter.
