@@ -20,8 +20,10 @@
 (define-module (gnu packages opencl)
   #:use-module (guix build-system gnu)
   #:use-module (guix build-system cmake)
+  #:use-module (guix build-system copy)
   #:use-module (guix build-system python)
   #:use-module (guix download)
+  #:use-module (guix utils)
   #:use-module (guix git-download)
   #:use-module ((guix licenses) #:prefix license:)
   #:use-module (guix packages)
@@ -47,44 +49,36 @@
 ;; tests that require such devices are all disabled.
 ;; Check https://lists.gnu.org/archive/html/guix-devel/2018-04/msg00293.html
 
+;; If you update either of opencl-headers, opencl-clhpp or opencl-icd-loader
+;; note that they are released together (lockstep) and must be updated
+;; together.
+(define-public opencl-headers
+  (package
+    (name "opencl-headers")
+    (version "2021.06.30")
+    (source
+      (origin
+        (method git-fetch)
+        (uri (git-reference
+              (url "https://github.com/KhronosGroup/OpenCL-Headers")
+              (commit (string-append "v" version))))
+        (file-name (git-file-name name version))
+        (sha256
+         (base32 "1nrvx0x9r1nz1qpmzbgffnn9h9pn2fwcxsksf101bkpmqziq5lii"))))
+    (build-system cmake-build-system)
+    (arguments `(#:tests? #f)) ; Not enabled during build.
+    (synopsis "The Khronos OpenCL headers")
+    (description
+     "This package provides the C headers by Khronos for OpenCL programming.")
+    (home-page "https://www.khronos.org/registry/OpenCL/")
+    (license license:asl2.0)))
+
 (define (make-opencl-headers major-version subversion)
-  (let ((commit "e986688daf750633898dfd3994e14a9e618f2aa5")
-        (revision "0"))
-    (package
-      (name "opencl-headers")
-      (version (git-version
-                (string-append major-version "." subversion ".0")
-                revision commit))
-      (source (origin
-                (method git-fetch)
-                (uri (git-reference
-                      (url "https://github.com/KhronosGroup/OpenCL-Headers")
-                      (commit commit)))
-                (file-name (git-file-name name version))
-                (sha256
-                 (base32
-                  "176ydpbyws5nr4av6hf8p41pkhc0rc4m4vrah9w6gp2fw2i32838"))))
-      (build-system gnu-build-system)
-      (arguments
-       `(#:phases
-         (modify-phases %standard-phases
-           (delete 'configure)
-           (delete 'build)
-           (delete 'check)
-           (replace 'install
-             (lambda* (#:key outputs #:allow-other-keys)
-               (copy-recursively (string-append "./opencl" (string-append
-                                                            ,major-version
-                                                            ,subversion) "/CL")
-                                 (string-append
-                                  (assoc-ref outputs "out")
-                                  "/include/CL")))))))
-      (synopsis "The Khronos OpenCL headers")
-      (description
-       "This package provides the C headers by Khronos for OpenCL
-programming.")
-      (home-page "https://www.khronos.org/registry/OpenCL/")
-      (license license:expat))))
+  ;; The upstream OpenCL-Headers repository is no longer separating headers by
+  ;; version; instead, you are supposed to #define CL_TARGET_OPENCL_VERSION.
+  (deprecated-package (string-append "opencl-headers-"
+                                     major-version "."
+                                     subversion) opencl-headers))
 
 (define-public opencl-headers-2.2
   (make-opencl-headers "2" "2"))
@@ -99,12 +93,10 @@ programming.")
 (define-public opencl-headers-1.0
   (make-opencl-headers "1" "0"))
 
-(define-public opencl-headers opencl-headers-2.2)
-
 (define-public opencl-clhpp
   (package
     (name "opencl-clhpp")
-    (version "2.0.10")
+    (version "2.0.15")
     (source
      (origin
        (method git-fetch)
@@ -112,21 +104,14 @@ programming.")
              (url "https://github.com/KhronosGroup/OpenCL-CLHPP")
              (commit (string-append "v" version))))
        (sha256
-        (base32 "0h5kpg5cl8wzfnqmv6i26aig2apv06ffm9p3rh35938n9r8rladm"))
+        (base32 "1wycdbvwbdn7lqdd3sby8471qg2zdisr70218ava6cfvxdsqcp83"))
        (file-name (git-file-name name version))))
     (native-inputs
      `(("python" ,python-wrapper)))
     (propagated-inputs
-     `(("opencl-headers" ,opencl-headers)))
+     (list opencl-headers))
     (arguments
-     `(#:configure-flags
-       (let ((out (assoc-ref %outputs "out")))
-         (list
-          "-DBUILD_EXAMPLES=OFF"
-          "-DBUILD_TESTS=OFF"
-          (string-append "-DCMAKE_INSTALL_PREFIX="
-                         (assoc-ref %outputs "out")
-                         "/include")))
+     `(#:configure-flags (list "-DBUILD_EXAMPLES=OFF" "-DBUILD_TESTS=OFF")
        ;; The regression tests require a lot more dependencies.
        #:tests? #f))
     (build-system cmake-build-system)
@@ -136,47 +121,52 @@ programming.")
      "This package provides the @dfn{host API} C++ headers for OpenCL.")
     (license license:expat)))
 
-(define-public ocl-icd
+(define-public opencl-icd-loader
   (package
-    (name "ocl-icd")
-    (version "2.2.12")
+    (name "opencl-icd-loader")
+    (version "2021.06.30")
     (source (origin
-              (method url-fetch)
-              (uri (string-append
-                    "https://forge.imag.fr/frs/download.php/836/ocl-icd-"
-                    version ".tar.gz"))
-              (file-name (string-append name "-" version ".tar.gz"))
+              (method git-fetch)
+              (uri (git-reference
+                    (url "https://github.com/KhronosGroup/OpenCL-ICD-Loader.git")
+                    (commit (string-append "v" version))))
+              (file-name (git-file-name name version))
               (sha256
                (base32
-                "1x2dr8p4dkfds56r38av360i3nv1y3326jmshxvjngaf6mlg6rbn"))
-              (modules '((guix build utils)))
-              (snippet
-               '(delete-file-recursively "khronos-headers"))))
-    (native-inputs
-     `(("opencl-headers" ,opencl-headers)
-       ("ruby" ,ruby)))
-    (inputs
-     `(("libgcrypt" ,libgcrypt)))
-    (build-system gnu-build-system)
-    (arguments
-     '(#:configure-flags '("DEBUG_OCL_ICD=1")))
+                "007ws357n1ijrxal1bf9lwy68p0dz1sm9cfcfnnz5f88iwc9xd6m"))))
+    (build-system cmake-build-system)
+    (arguments `(#:tests? #f)) ; Tests need stub loader setup.
     (native-search-paths
      (list (search-path-specification
-            (variable "OPENCL_VENDOR_PATH")
-            (files '("etc/OpenCL/vendors")))))
-    (search-paths native-search-paths)
-    (home-page "https://forge.imag.fr/projects/ocl-icd/")
-    (synopsis "OpenCL loader for Installable Client Drivers (ICDs)")
+            (variable "OCL_ICD_VENDORS")
+            (files '("etc/OpenCL/vendors"))
+            ;; Only supports a single directory.
+            (separator #f))))
+    (home-page "https://github.com/KhronosGroup/OpenCL-ICD-Loader")
+    (inputs (list opencl-headers))
+    (synopsis "OpenCL Installable Client Driver")
     (description
-     "OpenCL implementations are provided as ICDs (Installable Client
-Drivers).  An OpenCL program can use several ICDs thanks to the use of an ICD
-Loader as provided by this package.")
-    (license license:bsd-2)))
+     "OpenCL defines an Installable Client Driver (ICD) mechanism to allow
+developers to build applications against an Installable Client Driver loader
+(ICD loader) rather than linking their applications against a specific OpenCL
+implementation.  The ICD Loader is responsible for:
+
+@itemize
+@item Exporting OpenCL API entry points
+@item Enumerating OpenCL implementations
+@item Forwarding OpenCL API calls to the correct implementation
+@end itemize
+
+This package contains the Khronos official OpenCL ICD Loader.")
+    (license license:asl2.0)))
+
+(define-public ocl-icd
+  (deprecated-package "ocl-icd" opencl-icd-loader))
 
 (define-public clinfo
   (package
     (name "clinfo")
-    (version "2.2.18.04.06")
+    (version "3.0.21.02.21")
     (source
      (origin
        (method git-fetch)
@@ -185,26 +175,17 @@ Loader as provided by this package.")
              (commit version)))
        (file-name (git-file-name name version))
        (sha256
-        (base32 "0y2q0lz5yzxy970b7w7340vp4fl25vndahsyvvrywcrn51ipgplx"))))
+        (base32 "1sfxp6ai83i0vwdg7b05h0k07q6873q1z1avnyksj5zmzdnxya6j"))))
     (build-system gnu-build-system)
     (native-inputs
-     `(("opencl-headers" ,opencl-headers)))
+     (list opencl-headers))
     (inputs
-     `(("ocl-icd" ,ocl-icd)))
+     (list opencl-icd-loader))
     (arguments
-     `(#:phases
-       (modify-phases %standard-phases
-         (delete 'configure)
-         (replace 'build
-           (lambda _
-             (let ((cores (number->string (parallel-job-count))))
-               (setenv "CC" "gcc")
-               (invoke "make" "-j" cores))))
-         (replace 'install
-           (lambda* (#:key outputs #:allow-other-keys)
-             (invoke "make" "install" (string-append
-                                       "PREFIX="
-                                       (assoc-ref outputs "out"))))))
+     `(#:make-flags
+       (list ,(string-append "CC=" (cc-for-target))
+              (string-append "PREFIX=" (assoc-ref %outputs "out")))
+       #:phases (modify-phases %standard-phases (delete 'configure))
        #:tests? #f))
     (home-page "https://github.com/Oblomov/clinfo")
     (synopsis "Print information about OpenCL platforms and devices")
@@ -234,8 +215,7 @@ the system.")
         ;; There's a suspicious .isa binary file under kernels/.
         ;; Remove it.
         '(for-each delete-file (find-files "." "\\.isa$")))))
-    (native-inputs `(("pkg-config" ,pkg-config)
-                     ("python" ,python)))
+    (native-inputs (list pkg-config python))
     (inputs `(("clang@3.7" ,clang-3.7)
               ("clang-runtime@3.7" ,clang-runtime-3.7)
               ("glu" ,glu)
@@ -249,7 +229,7 @@ the system.")
               ("libxext" ,libxext)
               ("mesa-utils" ,mesa-utils)
               ("ncurses" ,ncurses)
-              ("ocl-icd" ,ocl-icd)
+              ("opencl-icd-loader" ,opencl-icd-loader)
               ("opencl-headers" ,opencl-headers)
               ("xextproto" ,xextproto)
               ("zlib" ,zlib)))
@@ -310,13 +290,9 @@ back-end for the LLVM compiler framework.")
        (file-name (git-file-name name version))))
     (build-system cmake-build-system)
     (native-inputs
-     `(("libltdl" ,libltdl)
-       ("pkg-config" ,pkg-config)))
+     (list libltdl pkg-config))
     (inputs
-     `(("clang" ,clang)
-       ("hwloc" ,hwloc-2 "lib")
-       ("llvm" ,llvm)
-       ("ocl-icd" ,ocl-icd)))
+     (list clang-9 llvm-9 `(,hwloc-2 "lib") opencl-icd-loader))
     (arguments
      `(#:configure-flags
        (list "-DENABLE_ICD=ON"
@@ -348,20 +324,17 @@ pocl.")
 (define-public python-pytools
   (package
     (name "python-pytools")
-    (version "2020.4")
+    (version "2021.2.7")
     (source
      (origin
        (method url-fetch)
        (uri (pypi-uri "pytools" version))
        (sha256
-        (base32 "0q7439iy365a874ckpavx6h1mhnnngfy8nl7xj5grdd127zkknrp"))))
+        (base32 "1yyr4k6sqx859gjhc02633l2vxwdnj6m2f5blmf7dgq0gzzgcf05"))))
     (build-system python-build-system)
+    (arguments `(#:tests? #f)) ; Tests depend on packages not present in Guix.
     (propagated-inputs
-     `(("python-appdirs" ,python-appdirs)
-       ("python-decorator" ,python-decorator)
-       ("python-numpy" ,python-numpy)
-       ("python-six" ,python-six)
-       ("python-mpi4py" ,python-mpi4py)))
+     (list python-appdirs python-numpy))
     (home-page "https://pypi.org/project/pytools/")
     (synopsis "Assorted tools for Python")
     (description
@@ -387,45 +360,25 @@ A lexer, @code{pytools.lex}.
 (define-public python-pyopencl
   (package
     (name "python-pyopencl")
-    (version "2019.1.1")
+    (version "2021.2.6")
     (source
      (origin
-       ;; The tarball on PyPI lacks test programs such as
-       ;; 'pygpu_language_opencl.cpp' so fetch it from Git.
-       ;; XXX: The server at git.tiker.net is unreliable.
        (method git-fetch)
        (uri (git-reference
-             (url "http://git.tiker.net/trees/pyopencl.git")
+             (url "https://github.com/inducer/pyopencl.git")
              (commit (string-append "v" version))
              (recursive? #t)))
        (file-name (git-file-name name version))
        (sha256
         (base32
-         "12q0rs8yla571vcfpsh0mfrjbdiayv0hi8r1rq0f178m3i3qjz80"))))
+         "1s2cls7avxvf753zzpx422ikslaxdnm8rz58zg7mal15yak0wv2x"))))
     (build-system python-build-system)
     (arguments
-     '(#:phases (modify-phases %standard-phases
-                  (add-before 'build 'set-home
-                    (lambda _
-                      ;; Some of the Python build scripts expect 'HOME' to be
-                      ;; set.
-                      (setenv "HOME" (getcwd))
-                      #t)))
-
-       ;; Tests in 'compyte/ndarray/setup_opencl.py' appear to rely on
-       ;; 'nvcc', which is not an option.
-       #:tests? #f))
+     `(#:tests? #f)) ; Tests cannot find pygpu_language_opencl.cpp
     (inputs
-     `(("opencl-headers" ,opencl-headers-1.2)   ;POCL only supports OpenCL 1.2
-       ("pybind11" ,pybind11)
-       ("ocl-icd" ,ocl-icd)))                     ;libOpenCL
+     (list opencl-headers pybind11 opencl-icd-loader))                     ;libOpenCL
     (propagated-inputs
-     `(("python-appdirs" ,python-appdirs)
-       ("python-decorator" ,python-decorator)
-       ("python-numpy" ,python-numpy)
-       ("python-pytools" ,python-pytools)
-       ("python-six" ,python-six)
-       ("python-mako" ,python-mako)))
+     (list python-appdirs python-numpy python-pytools python-mako))
     (home-page "http://mathema.tician.de/software/pyopencl")
     (synopsis "Python wrapper for OpenCL")
     (description
