@@ -14,6 +14,7 @@
 ;;; Copyright © 2020 Michael Rohleder <mike@rohleder.de>
 ;;; Copyright © 2020 Pierre Neidhardt <mail@ambrevar.xyz>
 ;;; Copyright © 2020 Marius Bakke <marius@gnu.org>
+;;; Copyright © 2021 Brice Waegeneire <brice@waegenei.re>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -47,9 +48,11 @@
   #:use-module (gnu packages audio)
   #:use-module (gnu packages autotools)
   #:use-module (gnu packages avahi)
-  #:use-module (gnu packages boost)
   #:use-module (gnu packages check)
-  #:use-module (gnu packages dbm)
+  #:use-module (gnu packages cpp)
+  #:use-module (gnu packages databases)
+  #:use-module (gnu packages documentation)
+  #:use-module (gnu packages freedesktop)
   #:use-module (gnu packages glib)
   #:use-module (gnu packages gettext)
   #:use-module (gnu packages gnome)
@@ -99,13 +102,10 @@
                    (("^/usr/bin/env") "env"))
                  #t))))
     (build-system gnu-build-system)
-    (inputs
-     `(("libvorbis" ,libvorbis)
-       ("libogg" ,libogg)
-       ("flac" ,flac)))
+    (propagated-inputs
+     (list flac libogg libvorbis opus))
     (native-inputs
-     `(("pkg-config" ,pkg-config)
-       ("python" ,python)))
+     (list pkg-config python))
     (home-page "http://www.mega-nerd.com/libsndfile/")
     (synopsis "Reading and writing files containing sampled sound")
     (description
@@ -134,11 +134,9 @@ for reading and writing new sound file formats.")
                "1ha46i0nbibq0pl0pjwcqiyny4hj8lp1bnl4dpxm64zjw9lb2zha"))))
     (build-system gnu-build-system)
     (native-inputs
-     `(("pkg-config" ,pkg-config)
-       ("automake" ,automake))) ;For up to date 'config.guess' and 'config.sub'.
+     (list pkg-config automake)) ;For up to date 'config.guess' and 'config.sub'.
     (propagated-inputs
-     `(("libsndfile" ,libsndfile)
-       ("fftw" ,fftw)))
+     (list libsndfile fftw))
     (arguments
      `(#:phases
        (modify-phases %standard-phases
@@ -180,7 +178,7 @@ rates.")
 (define-public pulseaudio
   (package
     (name "pulseaudio")
-    (version "14.0")
+    (version "15.0")
     (source (origin
              (method url-fetch)
              (uri (string-append
@@ -188,7 +186,7 @@ rates.")
                    name "-" version ".tar.xz"))
              (sha256
               (base32
-               "0qf20rgg0ysrnvg3359j56ndls07qmfn5rsy9r85bc42jdfpfd58"))
+               "1851rg4h6sjwanvd294hn52z321rc6vbs4gbfrlw53597dx8h2x4"))
              (modules '((guix build utils)))
              (snippet
               ;; Disable console-kit support by default since it's deprecated
@@ -201,14 +199,19 @@ rates.")
              (patches (search-patches
                        "pulseaudio-fix-mult-test.patch"
                        "pulseaudio-longer-test-timeout.patch"))))
-    (build-system gnu-build-system)
+    (build-system meson-build-system)
     (arguments
-     `(#:configure-flags (list "--localstatedir=/var" ;"--sysconfdir=/etc"
-                               "--disable-oss-output"
-                               "--enable-bluez5"
-                               (string-append "--with-udev-rules-dir="
-                                              (assoc-ref %outputs "out")
-                                              "/lib/udev/rules.d"))
+     `(#:configure-flags
+       (let ((out (assoc-ref %outputs "out")))
+         (list "-Doss-output=disabled"
+               "-Dlocalstatedir=/var"
+               (string-append "-Dudevrulesdir="
+                              out "/lib/udev/rules.d")
+               ;; Ensure the RUNPATH contains all installed library locations.
+               (string-append "-Dc_link_args=-Wl,-rpath="
+                              out "/lib/pulseaudio:"
+                              out "/lib:"
+                              out "/lib/pulse-" ,version "/modules")))
        #:phases (modify-phases %standard-phases
                  (add-before 'check 'pre-check
                    (lambda _
@@ -220,28 +223,28 @@ rates.")
                      (setenv "CK_DEFAULT_TIMEOUT" "120")
                      #t)))))
     (inputs
-     ;; TODO: Add optional inputs (GTK+?).
-     `(("alsa-lib" ,alsa-lib)
-       ("bluez" ,bluez)
-       ("sbc" ,sbc)
-       ("speexdsp" ,speexdsp)
-       ("libsndfile" ,libsndfile)
-       ("jack" ,jack-1) ; For routing the output to jack.
-       ("dbus" ,dbus)
-       ("glib" ,glib)
-       ("libltdl" ,libltdl)
-       ("fftwf" ,fftwf)
-       ("avahi" ,avahi)
-
-       ;; For the optional X11 modules.
-       ("libice" ,libice)
-       ("libsm" ,libsm)
-       ("libxcb" ,libxcb)
-       ("libxtst" ,libxtst)
-
-       ("eudev" ,eudev)))         ;for the detection of hardware audio devices
+     (list alsa-lib
+           bluez
+           sbc
+           speexdsp
+           libsndfile
+           jack-1 ; For routing the output to jack.
+           dbus
+           glib
+           libltdl
+           fftwf
+           avahi
+           webrtc-audio-processing
+           ;; For the optional X11 modules.
+           libice
+           libsm
+           libxcb
+           libxtst
+           elogind
+           eudev))         ;for the detection of hardware audio devices
     (native-inputs
      `(("check" ,check)
+       ("doxygen" ,doxygen)
        ("gettext" ,gettext-minimal)
        ("glib:bin" ,glib "bin")
        ("m4" ,m4)
@@ -249,9 +252,8 @@ rates.")
        ("perl-xml-parser" ,perl-xml-parser)
        ("pkg-config" ,pkg-config)))
     (propagated-inputs
-     ;; 'libpulse*.la' contain `-lgdbm' and `-lcap', so propagate them.
-     `(("libcap" ,libcap)
-       ("gdbm" ,gdbm)))
+     ;; 'libpulse*.la' contain `-ltdb' and `-lcap', so propagate them.
+     (list libcap tdb))
     (home-page "http://www.pulseaudio.org/")
     (synopsis "Sound server")
     (description
@@ -270,7 +272,7 @@ sound server.")
 (define-public pavucontrol
   (package
     (name "pavucontrol")
-    (version "4.0")
+    (version "5.0")
     (source (origin
              (method url-fetch)
              (uri (string-append
@@ -279,16 +281,16 @@ sound server.")
                    ".tar.xz"))
              (sha256
               (base32
-               "1qhlkl3g8d7h72xjskii3g1l7la2cavwp69909pzmbi2jyn5pi4g"))))
+               "0yjfiwpaydh5s8v3l78dhwhbsmcl1xsq3p8rvz80m9zinp1p4ayf"))))
     (build-system glib-or-gtk-build-system)
     (inputs
-     `(("adwaita-icon-theme" ,adwaita-icon-theme)          ;hard-coded theme
-       ("gtkmm" ,gtkmm)
-       ("libcanberra" ,libcanberra)
-       ("pulseaudio" ,pulseaudio)))
+     (list adwaita-icon-theme ;hard-coded theme
+           gtkmm-3
+           json-glib
+           libcanberra
+           pulseaudio))
     (native-inputs
-     `(("intltool" ,intltool)
-       ("pkg-config" ,pkg-config)))
+     (list intltool pkg-config))
     (home-page "https://www.freedesktop.org/software/pulseaudio/pavucontrol/")
     (synopsis "PulseAudio volume control")
     (description
@@ -323,9 +325,9 @@ easily control the volume of all clients, sinks, etc.")
                (("/usr") ""))))
          (delete 'configure)))) ; There's no configure phase.
     (inputs
-     `(("pulseaudio" ,pulseaudio)))
+     (list pulseaudio))
     (native-inputs
-     `(("pkg-config" ,pkg-config)))
+     (list pkg-config))
     (home-page "https://github.com/falconindy/ponymix")
     (synopsis "Console-based PulseAudio mixer")
     (description "Ponymix is a PulseAudio mixer and volume controller with a
@@ -358,69 +360,17 @@ sinks.")
                   (string-append pulse "/lib/libpulse.so.0")))
                #t))))))
     (inputs
-     `(("pulseaudio" ,pulseaudio)))
+     (list pulseaudio))
     (home-page "https://github.com/GeorgeFilipkin/pulsemixer/")
     (synopsis "Command-line and curses mixer for PulseAudio")
     (description "Pulsemixer is a PulseAudio mixer with command-line and
 curses-style interfaces.")
     (license l:expat)))
 
-(define-public pulseaudio-dlna
-  ;; The last release was in 2016; use a more recent commit.
-  (let ((commit "4472928dd23f274193f14289f59daec411023ab0")
-        (revision "1"))
-    (package
-      (name "pulseaudio-dlna")
-      (version (git-version "0.5.2" revision commit))
-      (source
-       (origin
-         (method git-fetch)
-         (uri (git-reference
-               (url "https://github.com/masmu/pulseaudio-dlna")
-               (commit commit)))
-         (file-name (git-file-name name version))
-         (sha256
-          (base32
-           "1dfn7036vrq49kxv4an7rayypnm5dlawsf02pfsldw877hzdamqk"))))
-      (build-system python-build-system)
-      (arguments `(#:python ,python-2))
-      (inputs
-       `(("python2-chardet" ,python2-chardet)
-         ("python2-dbus" ,python2-dbus)
-         ("python2-docopt" ,python2-docopt)
-         ("python2-futures" ,python2-futures)
-         ("python2-pygobject" ,python2-pygobject)
-         ("python2-lxml" ,python2-lxml)
-         ("python2-netifaces" ,python2-netifaces)
-         ("python2-notify2" ,python2-notify2)
-         ("python2-protobuf" ,python2-protobuf)
-         ("python2-psutil" ,python2-psutil)
-         ("python2-requests" ,python2-requests)
-         ("python2-pyroute2" ,python2-pyroute2)
-         ("python2-setproctitle" ,python2-setproctitle)
-         ("python2-zeroconf" ,python2-zeroconf)))
-      (home-page "https://github.com/masmu/pulseaudio-dlna")
-      (synopsis "Stream audio to DLNA/UPnP and Chromecast devices")
-      (description "This lightweight streaming server brings DLNA/UPnP and
-Chromecast support to PulseAudio.  It can stream your current PulseAudio
-playback to different UPnP devices (UPnP Media Renderers, including Sonos
-devices and some Smart TVs) or Chromecasts in your network.  You should also
-install one or more of the following packages alongside pulseaudio-dlna:
-
-@itemize
-@item ffmpeg - transcoding support for multiple codecs
-@item flac - FLAC transcoding support
-@item lame - MP3 transcoding support
-@item opus-tools - Opus transcoding support
-@item sox - WAV transcoding support
-@item vorbis-tools - Vorbis transcoding support
-@end itemize")
-      (license l:gpl3+))))
-
 (define-public pamixer
   (package
     (name "pamixer")
-    (version "1.4")
+    (version "1.6")
     (source
      (origin
        (method git-fetch)
@@ -429,22 +379,12 @@ install one or more of the following packages alongside pulseaudio-dlna:
              (commit version)))
        (file-name (git-file-name name version))
        (sha256
-        (base32 "1i14550n8paijwwnhksv5izgfqm3s5q2773bdfp6vyqybkll55f7"))))
-    (build-system gnu-build-system)
-    (arguments
-     `(#:tests? #f                      ; There is no test suite.
-       #:make-flags
-       (list (string-append "PREFIX=" (assoc-ref %outputs "out")))
-       #:phases
-       (modify-phases %standard-phases
-         (delete 'configure)            ; There's no configure phase.
-         (add-before 'install 'mkdir-bin
-           (lambda _
-             (mkdir-p (string-append (assoc-ref %outputs "out") "/bin"))
-             #t)))))
+        (base32 "0d0fcqv9fri1y2701lasscgmvljxzpyg95vy90b3d2ccdnqn3d1d"))))
+    (build-system meson-build-system)
+    (native-inputs
+     (list pkg-config))
     (inputs
-     `(("boost" ,boost)
-       ("pulseaudio" ,pulseaudio)))
+     (list cxxopts pulseaudio))
     (home-page "https://github.com/cdemoulins/pamixer")
     (synopsis "PulseAudio command line mixer")
     (description
@@ -475,15 +415,9 @@ volume levels of the sinks (get, set, decrease, increase, toggle mute, etc).")
              (delete-file "bootstrap.sh")
              #t)))))
     (native-inputs
-     `(("autoconf" ,autoconf)
-       ("automake" ,automake)
-       ("pkg-config" ,pkg-config)))
+     (list autoconf automake pkg-config))
     (inputs
-     `(("avahi" ,avahi)
-       ("gtk+" ,gtk+)
-       ("libnotify" ,libnotify)
-       ("libx11" ,libx11)
-       ("pulseaudio" ,pulseaudio)))
+     (list avahi gtk+ libnotify libx11 pulseaudio))
     (home-page "https://github.com/christophgysin/pasystray")
     (synopsis "PulseAudio controller for the system tray")
     (description "@command{pasystray} enables control of various
@@ -494,22 +428,19 @@ README.md for a detailed list of features.")
 (define-public paprefs
   (package
     (name "paprefs")
-    (version "1.1")
+    (version "1.2")
     (source
      (origin
        (method url-fetch)
        (uri (string-append "https://www.freedesktop.org/software/pulseaudio/"
-                           name "/" name "-" version ".tar.xz"))
+                           "paprefs/paprefs-" version ".tar.xz"))
        (sha256
-        (base32
-         "189z5p20hk0xv9vwvym293503j4pwl03xqk9hl7cl6dwgv0l7wkf"))))
+        (base32 "0czn71vc2j59ass1axr8ij7bh53wq2q0z4gw7xgd2dirvi01xwmk"))))
     (build-system meson-build-system)
     (native-inputs
-     `(("gettext" ,gettext-minimal)
-       ("pkg-config" ,pkg-config)))
+     (list gettext-minimal pkg-config))
     (inputs
-     `(("gtkmm" ,gtkmm)
-       ("pulseaudio" ,pulseaudio)))
+     (list gtkmm-3 pulseaudio))
     (home-page "https://freedesktop.org/software/pulseaudio/paprefs/")
     (synopsis "Simple GTK based configuration dialog for the PulseAudio sound
 server")
@@ -519,9 +450,9 @@ only configure local servers, and requires that a special module
 module-gsettings is loaded in the sound server.")
     (license l:gpl2)))
 
-(define-public rnnoise
+(define-public noise-suppression-for-voice
   (package
-    (name "rnnoise")
+    (name "noise-suppression-for-voice")
     (version "0.91")
     (source
      (origin
@@ -537,71 +468,62 @@ module-gsettings is loaded in the sound server.")
      ;; No tests.
      '(#:tests? #f))
     (inputs
-     `(;; TODO: Package VST to build the corresponding plugin.
-       ("pulseaudio" ,pulseaudio)))
+     (list ;; TODO: Package VST to build the corresponding plugin.
+           pulseaudio))
     (home-page "https://github.com/werman/noise-suppression-for-voice")
     (synopsis "Real-time Noise suppression plugin based on Xiph's RNNoise")
-    (description "The plugin is meant to suppress a wide range of noise
-origins: computer fans, office, crowd, airplane, car, train, construction.
+    (description "This plug-in is meant to suppress a wide range of noise
+origins: computer fans, offices, crowds, airplanes, cars, trains,
+construction, and more.
 
 Mild background noise is always suppressed, loud sounds, like
 clicking of mechanical keyboard, are suppressed while there is no voice
 however they are only reduced in volume when voice is present.
 
-The plugin is made to work with 1 or 2 channels (ladspa plugin),
+The plug-in is made to work with 1 or 2 channels (LADSPA plugin),
 16 bit, 48000 Hz audio input.")
     (license l:gpl3)))
 
 (define-public noisetorch
   (package
     (name "noisetorch")
-    (version "0.8.3")
+    (version "0.12.2")
     (source
      (origin
        (method git-fetch)
        (uri (git-reference
-             (url "https://github.com/lawl/NoiseTorch")
-             (commit version)))
+             (url "https://github.com/noisetorch/NoiseTorch")
+             (commit (string-append "v" version))))
        (file-name (git-file-name name version))
        (sha256
-        (base32 "115sq4waq048bv82lnq5sblf62m50yvyakx7x06qq359v7qf5ji1"))))
+        (base32 "1qwzqv4rww9xywkfnjr79489d35cypa1zm9rgm966g51zzwhxrck"))))
     (build-system go-build-system)
     (arguments
-     `(#:import-path "github.com/lawl/NoiseTorch"
+     `(#:import-path "github.com/noisetorch/NoiseTorch"
        #:install-source? #f
        #:phases
        (modify-phases %standard-phases
-         (add-after 'unpack 'symlink-rnnoise
+         (add-after 'unpack 'copy-rnnoise-library
            (lambda* (#:key inputs #:allow-other-keys)
-             (with-directory-excursion "src/github.com/lawl/NoiseTorch"
-               (let ((dir "librnnoise_ladspa/bin/ladspa")
-                     (rnnoise (assoc-ref inputs "rnnoise")))
+             (with-directory-excursion "src/github.com/noisetorch/NoiseTorch"
+               (let ((lib (search-input-file inputs
+                                             "lib/ladspa/librnnoise_ladspa.so"))
+                     (dir "c/ladspa"))
                  (mkdir-p dir)
-                 (symlink (string-append rnnoise "/lib/ladspa/librnnoise_ladspa.so")
-                          (string-append dir "/librnnoise_ladspa.so"))))
-             #t))
-         (add-after 'unpack 'gen-version.go
-           (lambda _
-             (with-directory-excursion "src/github.com/lawl/NoiseTorch"
-               (substitute* "main.go"
-                 (("//go:generate go run scripts/embedversion\\.go") ""))
-               (with-output-to-file "version.go"
-                 (lambda ()
-                   (format #t "package main~%~%var version=~s~&" ,version))))
-             #t))
+                 ;; Symlinking won't work: ‘cannot embed irregular file’!
+                 (copy-file lib (string-append dir "/rnnoise_ladspa.so"))))))
          (add-after 'unpack 'disable-update-check.go
            (lambda _
-             (with-directory-excursion "src/github.com/lawl/NoiseTorch"
+             (with-directory-excursion "src/github.com/noisetorch/NoiseTorch"
                (substitute* "main.go"
-                 ((".*updateCheck.*") "")))
-             #t))
+                 ((".*updateCheck.*") "")))))
          (add-before 'build 'go-generate
            (lambda _
-             (with-directory-excursion "src/github.com/lawl/NoiseTorch"
+             (with-directory-excursion "src/github.com/noisetorch/NoiseTorch"
                (invoke "go" "generate")))))))
     (inputs
-     `(("rnnoise" ,rnnoise)))
-    (home-page "https://github.com/lawl/NoiseTorch")
+     (list noise-suppression-for-voice))
+    (home-page "https://github.com/noisetorch/NoiseTorch")
     (synopsis "Real-time microphone noise suppression")
     (description "NoiseTorch creates a virtual PulseAudio microphone that
 suppresses noise, in any application.  Use whichever conferencing or VOIP
