@@ -3,6 +3,7 @@
 ;;; Copyright © 2018, 2019 Tobias Geerinckx-Rice <me@tobias.gr>
 ;;; Copyright © 2018 Efraim Flashner <efraim@flashner.co.il>
 ;;; Copyright © 2020 Marius Bakke <mbakke@fastmail.com>
+;;; Copyright © 2021 Mathieu Othacehe <othacehe@gnu.org>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -24,6 +25,7 @@
   #:use-module (guix packages)
   #:use-module (guix download)
   #:use-module (guix git-download)
+  #:use-module (guix gexp)
   #:use-module (guix build-system cmake)
   #:use-module (guix build-system gnu)
   #:use-module (guix build-system meson)
@@ -91,9 +93,8 @@ and for the GLSL.std.450 extended instruction set.
                                (string-append
                                 "-DSPIRV-Headers_SOURCE_DIR="
                                 (assoc-ref %build-inputs "spirv-headers")))))
-    (inputs `(("spirv-headers" ,spirv-headers)))
-    (native-inputs `(("pkg-config" ,pkg-config)
-                     ("python" ,python)))
+    (inputs (list spirv-headers))
+    (native-inputs (list pkg-config python))
     (home-page "https://github.com/KhronosGroup/SPIRV-Tools")
     (synopsis "API and commands for processing SPIR-V modules")
     (description
@@ -136,10 +137,8 @@ parser,disassembler, validator, and optimizer for SPIR-V.")
                 (string-append (assoc-ref inputs "spirv-tools") "/bin")))
              #t)))))
     (inputs
-     `(("glslang" ,glslang)
-       ("spirv-headers" ,spirv-headers)
-       ("spirv-tools" ,spirv-tools)))
-    (native-inputs `(("python" ,python)))
+     (list glslang spirv-headers spirv-tools))
+    (native-inputs (list python))
     (home-page "https://github.com/KhronosGroup/SPIRV-Cross")
     (synopsis "Parser for and converter of SPIR-V to other shader languages")
     (description
@@ -150,24 +149,25 @@ SPIR-V, aiming to emit GLSL or MSL that looks like human-written code.")
 (define-public glslang
   (package
     (name "glslang")
-    (version "8.13.3743")
+    (version "10-11.0.0")
     (source
      (origin
        (method git-fetch)
        (uri (git-reference
              (url "https://github.com/KhronosGroup/glslang")
-             (commit version)))
+             ;; Tag "10-11.0.0" was moved to "11.0.0".
+             ;; FIXME: Use (commit version) on next update.
+             (commit "11.0.0")))
        (sha256
         (base32
-         "0d20wfpp2fmbnz1hnsjr9xc62lxpj86ik2qyviqbni0pqj212cry"))
+         "14mn2awswl022ls75mfpsnpsl0ai0jgfbqj3sxcsqawyj5f432py"))
        (file-name (string-append name "-" version "-checkout"))))
     (build-system cmake-build-system)
     (arguments
      '(#:tests? #f                      ;FIXME: requires bundled SPIRV-Tools
        #:configure-flags '("-DBUILD_SHARED_LIBS=ON")))
     (native-inputs
-     `(("pkg-config" ,pkg-config)
-       ("python" ,python)))
+     (list pkg-config python))
     (home-page "https://github.com/KhronosGroup/glslang")
     (synopsis "OpenGL and OpenGL ES shader front end and validator")
     (description
@@ -183,7 +183,7 @@ interpretation of the specifications for these languages.")
 (define-public vulkan-headers
   (package
     (name "vulkan-headers")
-    (version "1.2.141")
+    (version "1.2.164")
     (source
      (origin
        (method git-fetch)
@@ -193,7 +193,7 @@ interpretation of the specifications for these languages.")
        (file-name (git-file-name name version))
        (sha256
         (base32
-         "10nmx6y4llllfcczyfz76amd0vkqv09dj952d19zkzmmgcval7zq"))))
+         "11wzxvwim4jna1yssbmprl211dhmz8vmrd498zww3bghzlj7bljv"))))
     (build-system cmake-build-system)
     (arguments
      `(#:tests? #f))                    ; No tests.
@@ -207,7 +207,7 @@ interpretation of the specifications for these languages.")
 (define-public vulkan-loader
   (package
     (name "vulkan-loader")
-    (version "1.2.140")
+    (version "1.2.162")
     (source
      (origin
        (method git-fetch)
@@ -217,10 +217,17 @@ interpretation of the specifications for these languages.")
        (file-name (git-file-name name version))
        (sha256
         (base32
-         "0rhyz0qgp0i7pcx6wlvgwy7j33d4cs0xx39f0b6igpfk0vk70r1w"))))
+         "15gx9ab6w1sjq9hkpbas7z2f8f47j6mlln6p3w26qmydjj8gfjjv"))))
     (build-system cmake-build-system)
     (arguments
-     `(#:phases (modify-phases %standard-phases
+     `(#:configure-flags
+       ,#~(list
+           (string-append "-DVULKAN_HEADERS_INSTALL_DIR="
+                          #$(this-package-input "vulkan-headers"))
+           (string-append "-DCMAKE_INSTALL_INCLUDEDIR="
+                          #$(this-package-input "vulkan-headers")
+                          "/include"))
+       #:phases (modify-phases %standard-phases
                   (add-after 'unpack 'unpack-googletest
                     (lambda* (#:key inputs #:allow-other-keys)
                       (let ((gtest (assoc-ref inputs "googletest:source")))
@@ -241,8 +248,9 @@ interpretation of the specifications for these languages.")
        ("libxrandr" ,libxrandr)
        ("pkg-config" ,pkg-config)
        ("python" ,python)
-       ("vulkan-headers" ,vulkan-headers)
        ("wayland" ,wayland)))
+    (inputs
+     (list vulkan-headers))
     (home-page
      "https://github.com/KhronosGroup/Vulkan-Loader")
     (synopsis "Khronos official ICD loader and validation layers for Vulkan")
@@ -261,7 +269,7 @@ and the ICD.")
 (define-public vulkan-tools
   (package
     (name "vulkan-tools")
-    (version "1.2.140")
+    (version "1.2.162")
     (source
      (origin
        (method git-fetch)
@@ -271,17 +279,12 @@ and the ICD.")
        (file-name (git-file-name name version))
        (sha256
         (base32
-         "08dk0q77kpycn4vv19jh3ig73gbq3psan246a7fss0nfxpiddg0j"))))
+         "129wzk7xj3vn3c8b4p7fzkd0npl58118s2i1d88gsfnlix54nagq"))))
     (build-system cmake-build-system)
     (inputs
-     `(("glslang" ,glslang)
-       ("libxrandr" ,libxrandr)
-       ("vulkan-loader" ,vulkan-loader)
-       ("wayland" ,wayland)))
+     (list glslang libxrandr vulkan-loader wayland))
     (native-inputs
-     `(("pkg-config" ,pkg-config)
-       ("python" ,python)
-       ("vulkan-headers" ,vulkan-headers)))
+     (list pkg-config python vulkan-headers))
     (arguments
      `(#:tests? #f                      ;no tests
        #:configure-flags (list (string-append "-DGLSLANG_INSTALL_DIR="
@@ -298,7 +301,7 @@ API.")
 (define-public shaderc
   (package
     (name "shaderc")
-    (version "2020.0")
+    (version "2020.4")
     (source
      (origin
        (method git-fetch)
@@ -308,12 +311,16 @@ API.")
        (file-name (git-file-name name version))
        (sha256
         (base32
-         "1kqqvsvib01bsmfbdy3fbwwpvkcdlfb6k71kjvzb3crql7w0rxff"))))
+         "07h78nd964h2bdm4drzws8i1gvyal8a3wlhbcm5qxqk6vknv8hrk"))))
     (build-system cmake-build-system)
     (arguments
      `(;; FIXME: Skip most of the tests, because enabling system gtest breaks
        ;; the build: <https://github.com/google/shaderc/issues/470>.
-       #:configure-flags '("-DSHADERC_SKIP_TESTS=ON")
+       #:configure-flags
+       (list "-DSHADERC_SKIP_TESTS=ON"
+             ;; Note: despite the name, this just specifies the headers.
+             (string-append "-Dglslang_SOURCE_DIR="
+                            (assoc-ref %build-inputs "glslang") "/include/glslang"))
        #:phases
        (modify-phases %standard-phases
          (add-after 'unpack 'do-not-look-for-bundled-sources
@@ -338,12 +345,9 @@ API.")
                          ,(package-version glslang))))
              #t)))))
     (inputs
-     `(("glslang" ,glslang)
-       ("python" ,python)
-       ("spirv-headers" ,spirv-headers)
-       ("spirv-tools" ,spirv-tools)))
+     (list glslang python spirv-headers spirv-tools))
     (native-inputs
-     `(("pkg-config" ,pkg-config)))
+     (list pkg-config))
     (home-page "https://github.com/google/shaderc")
     (synopsis "Tools for shader compilation")
     (description "Shaderc is a collection of tools, libraries, and tests for
@@ -351,10 +355,10 @@ shader compilation.")
     (license license:asl2.0)))
 
 (define-public vkd3d
-  (let ((commit "ecda316ef54d70bf1b3e860755241bb75873e53f")) ; Release 1.1.
+  (let ((commit "56cd4a94d541707959ce7677af6d1a34739e5579")) ; Release 1.2.
     (package
      (name "vkd3d")
-     (version "1.1")
+     (version "1.2")
      (source
       (origin
        (method git-fetch)
@@ -363,7 +367,7 @@ shader compilation.")
              (commit commit)))
        (sha256
         (base32
-         "05a28kspy8gzng181w28zjqdb3pj2ss83b0lwnppxbcdzsz7rvrf"))
+         "1n4a622drgnprvz5hjxzyzcsg2lp5rlf1sajki2vzf5gsx6fdpk8"))
        (file-name (string-append name "-" version "-checkout"))))
      (build-system gnu-build-system)
      (arguments
@@ -384,16 +388,16 @@ shader compilation.")
         ("libtool" ,libtool)
         ("pkg-config" ,pkg-config)))
      (inputs
-      `(("libx11" ,libx11)
-        ("libxcb" ,libxcb)
-        ("spirv-headers" ,spirv-headers)
-        ("spirv-tools" ,spirv-tools)
-        ("vulkan-headers" ,vulkan-headers)
-        ("vulkan-loader" ,vulkan-loader)
-        ("wine-minimal" ,wine-minimal) ; Needed for 'widl'.
-        ("xcb-util" ,xcb-util)
-        ("xcb-util-keysyms" ,xcb-util-keysyms)
-        ("xcb-util-wm" ,xcb-util-wm)))
+      (list libx11
+            libxcb
+            spirv-headers
+            spirv-tools
+            vulkan-headers
+            vulkan-loader
+            wine-minimal ; Needed for 'widl'.
+            xcb-util
+            xcb-util-keysyms
+            xcb-util-wm))
      (home-page "https://source.winehq.org/git/vkd3d.git/")
      (synopsis "Direct3D 12 to Vulkan translation library")
      (description "vkd3d is a library for translating Direct3D 12 to Vulkan.")
