@@ -2,7 +2,9 @@
 ;;; Copyright © 2012, 2013, 2017, 2018 Ludovic Courtès <ludo@gnu.org>
 ;;; Copyright © 2014 Eric Bavier <bavier@member.fsf.org>
 ;;; Copyright © 2016 Efraim Flashner <efraim@flashner.co.il>
-;;; Copyright © 2018 Tobias Geerinckx-Rice <me@tobias.gr>
+;;; Copyright © 2018, 2020 Tobias Geerinckx-Rice <me@tobias.gr>
+;;; Copyright © 2020 Marius Bakke <mbakke@fastmail.com>
+;;; Copyright © 2021 Matthew James Kraai <kraai@ftbfs.org>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -29,7 +31,9 @@
   #:use-module (gnu packages autotools)
   #:use-module (gnu packages bash)
   #:use-module (gnu packages bdw-gc)
+  #:use-module (gnu packages glib)
   #:use-module (gnu packages guile)
+  #:use-module (gnu packages gnome)
   #:use-module (gnu packages gnupg)
   #:use-module (gnu packages m4)
   #:use-module (gnu packages man)
@@ -41,33 +45,51 @@
 (define-public zile
   (package
     (name "zile")
-    (version "2.4.14")
+    (version "2.6.2")
     (source (origin
              (method url-fetch)
              (uri (string-append "mirror://gnu/zile/zile-"
                                  version ".tar.gz"))
              (sha256
               (base32
-               "0x3byaddms8l3g7igx6njycqsq98wgapysdb5c7lhcnajlkp8y3s"))))
+               "0hf788zadmwx0xp1dhrgqcfvhwnarh6h9b51va4dr2y9yfppvsvp"))))
     (build-system gnu-build-system)
     (arguments
-     '(#:phases
+     `(#:phases
        (modify-phases %standard-phases
          (add-before 'configure 'patch-/bin/sh
            (lambda* (#:key inputs #:allow-other-keys)
              (let ((bash (assoc-ref inputs "bash")))
                ;; Refer to the actual shell.
-               (substitute* '("lib/spawni.c" "src/funcs.c")
+               (substitute* '("src/shell.c")
                  (("/bin/sh")
                   (string-append bash "/bin/sh")))
-               #t))))))
+               #t)))
+         ;; Zile generates its manual pages by calling the built Zile
+         ;; with the --help argument.  That does not work when cross-
+         ;; compiling; use the native Zile added below in that case.
+         ,@(if (%current-target-system)
+               '((add-before 'build 'use-native-zile-for-documentation
+                   (lambda _
+                     (substitute* "build-aux/zile-help2man-wrapper"
+                       (("src/zile")
+                        (which "zile")))
+                     #t)))
+               '()))))
     (inputs
      `(("boehm-gc" ,libgc)
        ("ncurses" ,ncurses)
-       ("bash" ,bash)))
+       ("bash" ,bash)
+       ("gee" ,libgee)
+       ("glib" ,glib)))
     (native-inputs
      `(("perl" ,perl)
        ("help2man" ,help2man)
+       ;; When cross-compiling, Zile needs a native version of itself to
+       ;; generate the manual pages (see the related phase above).
+       ,@(if (%current-target-system)
+             `(("self" ,this-package))
+             '())
        ("pkg-config" ,pkg-config)))
     (home-page "https://www.gnu.org/software/zile/")
     (synopsis "Lightweight Emacs clone")
@@ -99,11 +121,11 @@ default Emacs configuration, but it carries a much lighter feature set.")
                   "0wlli8hqal9ikmbl3a49kyhzyf164jk6mdbir3bclq2gxszs532d"))
                 (file-name (string-append name "-" version "-checkout"))))
       (inputs
-       `(("guile" ,guile-2.0)
-         ,@(package-inputs zile)))
+       (modify-inputs (package-inputs zile)
+         (prepend guile-2.0)))
       (native-inputs
        `(("m4" ,m4)                               ;for 'bootstrap'
-         ("autoconf" ,autoconf-wrapper)
+         ("autoconf" ,autoconf)
          ("automake" ,automake)
 
          ;; For some reason, 'bootstrap' insists on having these.
@@ -127,6 +149,14 @@ default Emacs configuration, but it carries a much lighter feature set.")
                          "--gnulib-srcdir=gnulib"
                          "--skip-git" "--skip-po"
                          "--verbose")))
+             (replace 'patch-/bin/sh
+               (lambda* (#:key inputs #:allow-other-keys)
+                 (let ((bash (assoc-ref inputs "bash")))
+                   ;; Refer to the actual shell.
+                   (substitute* '("lib/spawni.c" "src/funcs.c")
+                     (("/bin/sh")
+                      (string-append bash "/bin/sh")))
+                   #t)))
              (add-after 'install 'wrap-command
                (lambda* (#:key outputs #:allow-other-keys)
                  ;; Add zile.scm to the search path.
