@@ -1,6 +1,6 @@
 ;;; GNU Guix --- Functional package management for GNU
 ;;; Copyright © 2015 Alex Kost <alezost@gmail.com>
-;;; Copyright © 2015 Ludovic Courtès <ludo@gnu.org>
+;;; Copyright © 2015, 2016, 2022 Ludovic Courtès <ludo@gnu.org>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -24,7 +24,10 @@
   #:use-module (guix gexp)
   #:use-module (guix records)
   #:use-module (ice-9 match)
-  #:export (lirc-service))
+  #:export (lirc-configuration
+            lirc-configuation?
+            lirc-service
+            lirc-service-type))
 
 ;;; Commentary:
 ;;;
@@ -35,7 +38,7 @@
 (define-record-type* <lirc-configuration>
   lirc-configuration make-lirc-configuration
   lirc-configuation?
-  (lirc          lirc-configuration-lirc          ;<package>
+  (lirc          lirc-configuration-lirc          ;file-like
                  (default lirc))
   (device        lirc-configuration-device)       ;string
   (driver        lirc-configuration-driver)       ;string
@@ -76,7 +79,9 @@
                  (list (service-extension shepherd-root-service-type
                                           lirc-shepherd-service)
                        (service-extension activation-service-type
-                                          (const %lirc-activation))))))
+                                          (const %lirc-activation))))
+                (description "Run LIRC, a daemon that decodes infrared signals
+from remote controls.")))
 
 (define* (lirc-service #:key (lirc lirc)
                        device driver config-file
