@@ -1,6 +1,7 @@
 ;;; GNU Guix --- Functional package management for GNU
-;;; Copyright © 2016, 2017, 2018, 2019, 2020 Ludovic Courtès <ludo@gnu.org>
+;;; Copyright © 2016, 2017, 2018, 2019, 2020, 2022 Ludovic Courtès <ludo@gnu.org>
 ;;; Copyright © 2018 Clément Lassieur <clement@lassieur.org>
+;;; Copyright © 2022 Maxim Cournoyer <maxim.cournoyer@gmail.com>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -19,7 +20,9 @@
 
 (define-module (gnu tests base)
   #:use-module (gnu tests)
+  #:use-module (gnu image)
   #:use-module (gnu system)
+  #:autoload   (gnu system image) (system-image)
   #:use-module (gnu system shadow)
   #:use-module (gnu system nss)
   #:use-module (gnu system vm)
@@ -33,19 +36,22 @@
   #:use-module (gnu packages base)
   #:use-module (gnu packages bash)
   #:use-module (gnu packages imagemagick)
+  #:use-module (gnu packages linux)
   #:use-module (gnu packages ocr)
   #:use-module (gnu packages package-management)
-  #:use-module (gnu packages linux)
   #:use-module (gnu packages tmux)
+  #:use-module (gnu packages virtualization)
   #:use-module (guix gexp)
   #:use-module (guix store)
   #:use-module (guix monads)
+  #:use-module (guix modules)
   #:use-module (guix packages)
-  #:use-module (srfi srfi-1)
+  #:use-module ((srfi srfi-1) #:hide (partition))
   #:use-module (ice-9 match)
   #:export (run-basic-test
             %test-basic-os
             %test-halt
+            %test-root-unmount
             %test-cleanup
             %test-mcron
             %test-nss-mdns))
@@ -87,6 +93,7 @@ Otherwise assume that there is no password for root."
           (use-modules (gnu build marionette)
                        (guix build syscalls)
                        (srfi srfi-1)
+                       (srfi srfi-19)
                        (srfi srfi-26)
                        (srfi srfi-64)
                        (ice-9 match))
@@ -94,9 +101,7 @@ Otherwise assume that there is no password for root."
           (define marionette
             (make-marionette #$command))
 
-          (mkdir #$output)
-          (chdir #$output)
-
+          (test-runner-current (system-test-runner #$output))
           (test-begin "basic")
 
           #$(and initialization
@@ -198,6 +203,16 @@ info --version")
                      (pk 'services services)
                      '(root #$@(operating-system-shepherd-service-names os)))))
 
+          (test-equal "libc honors /etc/localtime"
+            -7200          ;CEST = GMT+2
+            ;; Assume OS is configured to have a CEST timezone.
+            (let* ((sept-2021 (time-second
+                               (date->time-utc
+                                (make-date 0 0 00 12 01 09 2021 7200)))))
+              (marionette-eval
+               `(tm:gmtoff (localtime ,sept-2021))
+               marionette)))
+
           (test-equal "/var/log/messages is not world-readable"
             #o640                                ;<https://bugs.gnu.org/40405>
             (begin
@@ -327,7 +342,7 @@ info --version")
                       (wait-for-screen-text marionette
                                             (lambda (text)
                                               (string-contains text "Password"))
-                                            #:ocrad
+                                            #:ocr
                                             #$(file-append ocrad "/bin/ocrad"))
                       (marionette-type (string-append password "\n\n")
                                        marionette))
@@ -337,7 +352,8 @@ info --version")
               ;; It can take a while before the shell commands are executed.
               (marionette-eval '(use-modules (rnrs io ports)) marionette)
               (wait-for-file "/root/logged-in" marionette
-                             #:read 'get-string-all)))
+                             #:read 'get-string-all
+                             #:timeout 30)))
 
           (test-equal "getlogin on tty1"
             "\"root\""
@@ -351,7 +367,8 @@ info --version")
               ;; It can take a while before the shell commands are executed.
               (marionette-eval '(use-modules (rnrs io ports)) marionette)
               (wait-for-file "/root/login-id" marionette
-                             #:read 'get-string-all)))
+                             #:read 'get-string-all
+                             #:timeout 30)))
 
           ;; There should be one utmpx entry for the user logged in on tty1.
           (test-equal "utmpx entry"
@@ -407,6 +424,12 @@ info --version")
               (x
                (pk 'failure x #f))))
 
+          (test-assert "nscd configuration action"
+            (marionette-eval '(with-shepherd-action 'nscd ('configuration)
+                                                    results
+                                (file-exists? (car results)))
+                             marionette))
+
           (test-equal "nscd invalidate action"
             '(#t)                                 ;one value, #t
             (marionette-eval '(with-shepherd-action 'nscd ('invalidate "hosts")
@@ -486,27 +509,27 @@ info --version")
 
           (test-assert "screendump"
             (begin
-              (marionette-control (string-append "screendump " #$output
-                                                 "/tty1.ppm")
-                                  marionette)
-              (file-exists? "tty1.ppm")))
+              (let ((capture
+                     (string-append #$output "/tty1.ppm")))
+                (marionette-control
+                 (string-append "screendump " capture) marionette)
+                (file-exists? capture))))
 
           (test-assert "screen text"
-            (let ((text (marionette-screen-text marionette
-                                                #:ocrad
-                                                #$(file-append ocrad
-                                                               "/bin/ocrad"))))
-              ;; Check whether the welcome message and shell prompt are
-              ;; displayed.  Note: OCR confuses "y" and "V" for instance, so
-              ;; we cannot reliably match the whole text.
-              (and (string-contains text "This is the GNU")
-                   (string-contains text
-                                    (string-append
-                                     "root@"
-                                     #$(operating-system-host-name os))))))
+            (wait-for-screen-text
+             marionette
+             (lambda (text)
+               ;; Check whether the welcome message and shell prompt are
+               ;; displayed.  Note: OCR confuses "y" and "V" for instance, so
+               ;; we cannot reliably match the whole text.
+               (and (string-contains text "This is the GNU")
+                    (string-contains text
+                                     (string-append
+                                      "root@"
+                                      #$(operating-system-host-name os)))))
+             #:ocr #$(file-append ocrad "/bin/ocrad")))
 
-          (test-end)
-          (exit (= (test-runner-fail-count (test-runner-current)) 0)))))
+          (test-end))))
 
   (gexp->derivation name test))
 
@@ -604,6 +627,145 @@ in a loop.  See <http://bugs.gnu.org/26931>.")
                                     (guix combinators)))))
       (run-halt-test (virtual-machine os))))))
 
+
+;;;
+;;; Root cleanly unmounted.
+;;;
+
+(define (run-root-unmount-test os)
+  (define test-image
+    (image (operating-system os)
+           (format 'compressed-qcow2)
+           (volatile-root? #f)
+           (shared-store? #f)
+           (partition-table-type 'mbr)
+           (partitions
+            (list (partition
+                   (size 'guess)
+                   (offset (* 512 2048))          ;leave room for GRUB
+                   (flags '(boot))
+                   (label "root-under-test")))))) ;max 16 characters!
+
+  (define observer-os
+    (marionette-operating-system
+     %simple-os
+     #:imported-modules
+     (source-module-closure '((guix build syscalls)
+                              (gnu build file-systems)))))
+
+  (define test
+    (with-imported-modules (source-module-closure
+                            '((gnu build marionette)
+                              (guix build utils)))
+      #~(begin
+          (use-modules (gnu build marionette)
+                       (guix build utils)
+                       (srfi srfi-64)
+                       (ice-9 ftw))
+
+          (define image
+            "/tmp/writable-image.qcow2")
+
+          (define (test-system-marionette)
+            ;; Return a marionette on a system where we'll run 'halt'.
+            (invoke #$(file-append qemu-minimal "/bin/qemu-img")
+                    "create" "-f" "qcow2" image "3G"
+                    "-b" #$(system-image test-image) "-F" "qcow2")
+            (make-marionette
+             `(,(string-append #$qemu-minimal "/bin/" (qemu-command))
+               ,@(if (file-exists? "/dev/kvm")
+                     '("-enable-kvm")
+                     '())
+               "-no-reboot"
+               "-m" "1024"                        ;memory size, in MiB
+               "-drive" ,(format #f "file=~a,if=virtio" image))))
+
+          (define witness-size
+            ;; Size of the /witness file.
+            (* 20 (expt 2 20)))
+
+          (test-runner-current (system-test-runner #$output))
+          (test-begin "root-unmount")
+
+          (let ((marionette (test-system-marionette)))
+            (test-assert "file created"
+              (marionette-eval `(begin
+                                  (use-modules (guix build utils))
+                                  (call-with-output-file "/witness"
+                                    (lambda (port)
+                                      (call-with-input-file "/dev/random"
+                                        (lambda (input)
+                                          (dump-port input port
+                                                     ,witness-size))))))
+                               marionette))
+
+            ;; Halt the system.
+            (marionette-eval '(system* "/run/current-system/profile/sbin/halt")
+                             marionette)
+
+            (display "waiting for marionette to complete...")
+            (force-output)
+            (false-if-exception (waitpid (marionette-pid marionette)))
+            (display " done\n")
+            (force-output))
+
+          ;; Remove the sockets used by the marionette above to avoid
+          ;; EADDRINUSE.
+          (for-each delete-file
+                    (find-files "/tmp" (lambda (file stat)
+                                         (eq? (stat:type stat) 'socket))))
+
+          ;; Now boot another system and check whether the root file system of
+          ;; the first one was cleanly unmounted.
+
+          (let ((observer
+                 (make-marionette (list #$(virtual-machine observer-os)
+                                        "-drive"
+                                        (format #f "file=~a,if=virtio" image)))))
+            (test-assert "partitions"
+              (marionette-eval '(begin
+                                  (use-modules (gnu build file-systems))
+                                  (disk-partitions))
+                               observer))
+
+            (test-assert "partition found"
+              (marionette-eval '(find-partition-by-label "root-under-test")
+                               observer))
+
+            (test-assert "root file system is clean"
+              (marionette-eval '(cleanly-unmounted-ext2?
+                                 (find-partition-by-label "root-under-test"))
+                               observer))
+
+            (test-equal "root file system contains /witness"
+              witness-size
+              (let ((files (marionette-eval
+                            '(begin
+                               (use-modules (guix build syscalls)
+                                            (ice-9 ftw))
+                               (mount (find-partition-by-label "root-under-test")
+                                      "/mnt" "ext4" MS_RDONLY)
+                               (scandir "/mnt"))
+                            observer)))
+                (if (member "witness" files)
+                    (marionette-eval '(stat:size (stat "/mnt/witness"))
+                                     observer)
+                    files))))
+
+          (test-end))))
+
+  (gexp->derivation "root-unmount" test))
+
+(define %test-root-unmount
+  (system-test
+   (name "root-unmount")
+   (description
+    "Make sure the root file system is cleanly unmounted when the system is
+halted.")
+   (value
+    (let ((os (marionette-operating-system %simple-os)))
+      (run-root-unmount-test os)))))
+
 
 ;;;
 ;;; Cleanup of /tmp, /var/run, etc.
@@ -642,9 +804,7 @@ in a loop.  See <http://bugs.gnu.org/26931>.")
           (define marionette
             (make-marionette (list #$(virtual-machine os))))
 
-          (mkdir #$output)
-          (chdir #$output)
-
+          (test-runner-current (system-test-runner #$output))
           (test-begin "cleanup")
 
           (test-assert "dirty service worked"
@@ -657,8 +817,7 @@ in a loop.  See <http://bugs.gnu.org/26931>.")
                                 (scandir "/tmp"))
                              marionette))
 
-          (test-end)
-          (exit (= (test-runner-fail-count (test-runner-current)) 0)))))
+          (test-end))))
 
   (gexp->derivation "cleanup" test))
 
@@ -713,9 +872,7 @@ non-ASCII names from /tmp.")
           (define marionette
             (make-marionette (list #$(virtual-machine os))))
 
-          (mkdir #$output)
-          (chdir #$output)
-
+          (test-runner-current (system-test-runner #$output))
           (test-begin "mcron")
 
           (test-assert "service running"
@@ -752,8 +909,7 @@ non-ASCII names from /tmp.")
                                 result)
                              marionette))
 
-          (test-end)
-          (exit (= (test-runner-fail-count (test-runner-current)) 0)))))
+          (test-end))))
 
   (gexp->derivation name test))
 
@@ -824,6 +980,7 @@ non-ASCII names from /tmp.")
           (mkdir #$output)
           (chdir #$output)
 
+          (test-runner-current (system-test-runner))
           (test-begin "avahi")
 
           (test-assert "nscd PID file is created"
@@ -902,8 +1059,7 @@ non-ASCII names from /tmp.")
                     (= (hostent:addrtype result) AF_INET)))))
 
 
-          (test-end)
-          (exit (= (test-runner-fail-count (test-runner-current)) 0)))))
+          (test-end))))
 
   (gexp->derivation "nss-mdns" test))
 
