@@ -4,6 +4,7 @@
 ;;; Copyright © 2015 Federico Beffa <beffa@fbengineering.ch>
 ;;; Copyright © 2016 Alex Griffin <a@ajgrf.com>
 ;;; Copyright © 2016 David Craven <david@craven.ch>
+;;; Copyright © 2016, 2017, 2020, 2022 Efraim Flashner <efraim@flashner.co.il>
 ;;; Copyright © 2017 Tobias Geerinckx-Rice <me@tobias.gr>
 ;;; Copyright © 2019 Guy Fleury Iteriteka <hoonandon@gmail.com>
 ;;;
@@ -35,6 +36,8 @@
   #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-26)
   #:export (%mirrors
+            %disarchive-mirrors
+            %download-fallback-test
             (url-fetch* . url-fetch)
             url-fetch/executable
             url-fetch/tarbomb
@@ -87,23 +90,21 @@
        "ftp://ftp.ring.gr.jp/pub/net/gnupg/"
        "ftp://ftp.gnupg.org/gcrypt/")
       (gnome
-       "http://ftp.belnet.be/ftp.gnome.org/"
-       "http://ftp.linux.org.uk/mirrors/ftp.gnome.org/"
-       "http://ftp.gnome.org/pub/GNOME/"
        "https://download.gnome.org/"
+       "http://ftp.gnome.org/pub/GNOME/"
        "http://mirror.yandex.ru/mirrors/ftp.gnome.org/")
       (hackage
        "http://hackage.haskell.org/")
       (savannah           ; http://download0.savannah.gnu.org/mirmon/savannah/
-       "http://download.savannah.gnu.org/releases/"
-       "http://nongnu.freemirror.org/nongnu/"
-       "http://ftp.cc.uoc.gr/mirrors/nongnu.org/"
-       "http://ftp.twaren.net/Unix/NonGNU/"
-       "http://mirror.csclub.uwaterloo.ca/nongnu/"
-       "http://nongnu.askapache.com/"
-       "http://savannah.c3sl.ufpr.br/"
+       "https://download.savannah.gnu.org/releases/"
+       "https://de.freedif.org/savannah/"
+       "https://ftp.cc.uoc.gr/mirrors/nongnu.org/"
+       "http://ftp.twaren.net/Unix/NonGNU/" ; https appears unsupported
+       "https://mirror.csclub.uwaterloo.ca/nongnu/"
+       "https://nongnu.askapache.com/"
+       "https://savannah.c3sl.ufpr.br/"
        "http://download.savannah.gnu.org/releases-noredirect/"
-       "http://download-mirror.savannah.gnu.org/releases/"
+       "https://download-mirror.savannah.gnu.org/releases/"
        "ftp://ftp.twaren.net/Unix/NonGNU/"
        "ftp://mirror.csclub.uwaterloo.ca/nongnu/"
        "ftp://mirror.publicns.net/pub/nongnu/"
@@ -137,6 +138,7 @@
        "http://kernel.osuosl.org/pub/"
        "http://ftp.be.debian.org/pub/"
        "http://mirror.linux.org.au/"
+       "https://mirrors.edge.kernel.org/pub/"
        "ftp://ftp.funet.fi/pub/mirrors/ftp.kernel.org/pub/")
       (apache             ; from http://www.apache.org/mirrors/dist.html
        "http://www.eu.apache.org/dist/"
@@ -245,6 +247,11 @@
        "http://cran.stat.auckland.ac.nz/"
        "http://cran.mirror.ac.za/"
        "http://cran.csie.ntu.edu.tw/")
+      (ctan
+       ;; This is the CTAN mirror multiplexor service, which automatically
+       ;; redirect to a mirror in or close to the country of the requester
+       ;; (see: https://ctan.org/mirrors/).
+       "https://mirror.ctan.org/")
       (imagemagick
        ;; from http://www.imagemagick.org/script/download.php
        ;; (without mirrors that are unavailable or not up to date)
@@ -261,51 +268,57 @@
        "http://ftp.debian.org/debian/"
        "http://archive.debian.org/debian/")
       (kde
-       "http://download.kde.org"
-       "http://download.kde.org/Attic" ; for when it gets archived.
-       ;; Mirrors from http://files.kde.org/extra/mirrors.html
+       "https://download.kde.org/"
+       "https://download.kde.org/Attic/"    ; for when it gets archived.
+       ;; I could not find the classic static mirror list anymore.  Instead,
+       ;; add ‘.mirrorlist’ to the end of a recent download.kde.org tarball URL.
        ;; Europe
-       "http://mirror.easyname.at/kde"
-       "http://mirror.karneval.cz/pub/kde"
-       "http://ftp.fi.muni.cz/pub/kde/"
-       "http://mirror.oss.maxcdn.com/kde/"
-       "http://ftp5.gwdg.de/pub/linux/kde/"
-       "http://ftp-stud.fht-esslingen.de/Mirrors/ftp.kde.org/pub/kde/"
-       "http://mirror.klaus-uwe.me/kde/ftp/"
-       "http://kde.beta.mirror.ga/"
-       "http://kde.alpha.mirror.ga/"
-       "http://mirror.netcologne.de/kde"
-       "http://vesta.informatik.rwth-aachen.de/ftp/pub/mirror/kde/"
-       "http://ftp.rz.uni-wuerzburg.de/pub/unix/kde/"
-       "http://mirrors.dotsrc.org/kde/"
-       "http://ftp.funet.fi/pub/mirrors/ftp.kde.org/pub/kde/"
-       "http://kde-mirror.freenux.org/"
-       "http://mirrors.ircam.fr/pub/KDE/"
-       "http://www-ftp.lip6.fr/pub/X11/kde/"
-       "http://fr2.rpmfind.net/linux/KDE/"
+       "https://mirrors.xtom.de/kde/"
+       "https://mirror.lyrahosting.com/pub/kde/"
+       "https://mirrors.xtom.nl/kde/"
+       "https://mirror.hs-esslingen.de/Mirrors/ftp.kde.org/pub/kde/"
+       "https://mirror.kumi.systems/kde/ftp/"
+       "https://mirrors.ircam.fr/pub/KDE/"
+       "https://ftp.gwdg.de/pub/linux/kde/"
+       "https://mirrors.gethosted.online/kde/pub/kde/"
+       "https://fr2.rpmfind.net/linux/KDE/"
+       "https://mirror.faigner.de/kde/ftp/"
+       "https://www.mirrorservice.org/sites/download.kde.org/"
+       "https://mirrors.ukfast.co.uk/sites/kde.org/ftp/"
+       "https://mirrors.dotsrc.org/kde/"
        "http://kde.mirror.anlx.net/"
-       "http://www.mirrorservice.org/sites/ftp.kde.org/pub/kde/"
-       "http://ftp.heanet.ie/mirrors/ftp.kde.org/"
-       "http://ftp.nluug.nl/pub/windowing/kde/"
-       "http://ftp.surfnet.nl/windowing/kde/"
-       "http://ftp.icm.edu.pl/pub/unix/kde/"
-       "http://ftp.pbone.net/pub/kde/"
-       "http://piotrkosoft.net/pub/mirrors/ftp.kde.org/"
-       "http://mirrors.fe.up.pt/pub/kde/"
-       "http://ftp.iasi.roedu.net/pub/mirrors/ftp.kde.org/"
-       "http://ftp.acc.umu.se/mirror/kde.org/ftp/"
-       "http://kde.ip-connect.vn.ua/"
+       "https://mirror.karneval.cz/pub/kde/"
+       "https://ftp.fi.muni.cz/pub/kde/"
+       "https://www-ftp.lip6.fr/pub/X11/kde/"
+       "https://ftp.icm.edu.pl/pub/unix/kde/"
+       "https://kde.mirror.garr.it/kde/ftp/"
+       "https://ftp.acc.umu.se/mirror/kde.org/ftp/"
+       "https://mirrors.up.pt/pub/kde/"
+       "https://mirrors.nav.ro/kde/"
+       "https://mirrors.xtom.ee/kde/"
+       "https://ftp.funet.fi/pub/mirrors/ftp.kde.org/pub/kde/"
+       "https://kde.ip-connect.vn.ua/"
+       "https://mirrors.netix.net/kde/"
+       "https://ftp.cc.uoc.gr/mirrors/kde/"
        ;; North America
-       "http://mirror.its.dal.ca/kde/"
-       "http://mirror.csclub.uwaterloo.ca/kde/"
-       "http://mirror.cc.columbia.edu/pub/software/kde/"
-       "http://kde.mirrors.hoobly.com/"
-       "http://ftp.ussg.iu.edu/kde/"
-       "http://mirrors.mit.edu/kde/"
-       "http://kde.mirrors.tds.net/pub/kde/"
+       "https://mirror.its.dal.ca/kde/"
+       "https://nnenix.mm.fcix.net/kdeftp/"
+       "https://mirrors.mit.edu/kde/"
+       "https://mirror.csclub.uwaterloo.ca/kde/"
+       "https://mirror.fcix.net/kdeftp/"
+       "https://mirrors.ocf.berkeley.edu/kde/"
+       "https://mirrors.xtom.com/kde/"
+       ;; South America
+       "https://kde.c3sl.ufpr.br/"
+       ;; Asia
+       "https://mirrors.bfsu.edu.cn/kde/"
+       "https://ftp-srv2.kddi-research.jp/pub/X11/kde/"
+       "https://mirrors.xtom.jp/kde/"
+       "https://mirrors.xtom.hk/kde/"
+       ;; Africa
+       "http://mirror.retentionrange.co.bw/kde/"
        ;; Oceania
-       "http://ftp.kddlabs.co.jp/pub/X11/kde/"
-       "http://kde.mirror.uber.com.au/")
+       "https://mirrors.xtom.au/kde/")
       (openbsd
        "https://ftp.openbsd.org/pub/OpenBSD/"
        ;; Anycast CDN redirecting to your friendly local mirror.
@@ -370,7 +383,7 @@
   ;; procedure that takes a file name, an algorithm (symbol) and a hash
   ;; (bytevector), and returns a URL or #f.
   '(begin
-     (use-modules (guix base32))
+     (use-modules (guix base16) (guix base32))
 
      (define (guix-publish host)
        (lambda (file algo hash)
@@ -380,12 +393,6 @@
                         file "/" (symbol->string algo) "/"
                         (bytevector->nix-base32-string hash))))
 
-     ;; XXX: (guix base16) appeared in March 2017 (and thus 0.13.0) so old
-     ;; installations of the daemon might lack it.  Thus, load it lazily to
-     ;; avoid gratuitous errors.  See <https://bugs.gnu.org/33542>.
-     (module-autoload! (current-module)
-                       '(guix base16) '(bytevector->base16-string))
-
      (list (guix-publish "ci.guix.gnu.org")
            (lambda (file algo hash)
              ;; 'tarballs.nixos.org' supports several algorithms.
@@ -406,12 +413,30 @@
   (plain-file "content-addressed-mirrors"
               (object->string %content-addressed-mirrors)))
 
+(define %no-mirrors-file
+  ;; File specifying an empty list of mirrors, for fallback tests.
+  (plain-file "no-content-addressed-mirrors" (object->string ''())))
+
+(define %disarchive-mirrors
+  ;; TODO: Eventually turn into a procedure that takes a hash algorithm
+  ;; (symbol) and hash (bytevector).
+  '("https://disarchive.guix.gnu.org/"
+    "https://disarchive.ngyro.com/"))
+
+(define %disarchive-mirror-file
+  (plain-file "disarchive-mirrors" (object->string %disarchive-mirrors)))
+
+(define %no-disarchive-mirrors-file
+  ;; File specifying an empty list of Disarchive mirrors, for fallback tests.
+  (plain-file "no-disarchive-mirrors" (object->string '())))
+
 (define built-in-builders*
   (store-lift built-in-builders))
 
 (define* (built-in-download file-name url
                             #:key system hash-algo hash
                             mirrors content-addressed-mirrors
+                            disarchive-mirrors
                             executable?
                             (guile 'unused))
   "Download FILE-NAME from URL using the built-in 'download' builder.  When
@@ -422,13 +447,16 @@ explicitly depend on Guile, GnuTLS, etc.  Instead, the daemon performs the
 download by itself using its own dependencies."
   (mlet %store-monad ((mirrors (lower-object mirrors))
                       (content-addressed-mirrors
-                       (lower-object content-addressed-mirrors)))
+                       (lower-object content-addressed-mirrors))
+                      (disarchive-mirrors (lower-object disarchive-mirrors)))
     (raw-derivation file-name "builtin:download" '()
                     #:system system
                     #:hash-algo hash-algo
                     #:hash hash
                     #:recursive? executable?
-                    #:sources (list mirrors content-addressed-mirrors)
+                    #:sources (list mirrors
+                                    content-addressed-mirrors
+                                    disarchive-mirrors)
 
                     ;; Honor the user's proxy and locale settings.
                     #:leaked-env-vars '("http_proxy" "https_proxy"
@@ -439,6 +467,7 @@ download by itself using its own dependencies."
                                  ("mirrors" . ,mirrors)
                                  ("content-addressed-mirrors"
                                   . ,content-addressed-mirrors)
+                                 ("disarchive-mirrors" . ,disarchive-mirrors)
                                  ,@(if executable?
                                        '(("executable" . "1"))
                                        '()))
@@ -449,6 +478,24 @@ download by itself using its own dependencies."
                     ;; for that built-in is widespread.
                     #:local-build? #t)))
 
+(define %download-fallback-test
+  ;; Define whether to test one of the download fallback mechanism.  Possible
+  ;; values are:
+  ;;
+  ;;   - #f, to use the normal download methods, not trying to exercise the
+  ;;     fallback mechanism;
+  ;;
+  ;;   - 'none, to disable all the fallback mechanisms;
+  ;;
+  ;;   - 'content-addressed-mirrors, to purposefully attempt to download from
+  ;;     a content-addressed mirror;
+  ;;
+  ;;   - 'disarchive-mirrors, to download from Disarchive + Software Heritage.
+  ;;
+  ;; This is meant to be used for testing purposes.
+  (make-parameter (and=> (getenv "GUIX_DOWNLOAD_FALLBACK_TEST")
+                         string->symbol)))
+
 (define* (url-fetch* url hash-algo hash
                      #:optional name
                      #:key (system (%current-system))
@@ -484,7 +531,10 @@ name in the store."
           (unless (member "download" builtins)
             (error "'guix-daemon' is too old, please upgrade" builtins))
 
-          (built-in-download (or name file-name) url
+          (built-in-download (or name file-name)
+                             (match (%download-fallback-test)
+                               ((or #f 'none) url)
+                               (_ "https://example.org/does-not-exist"))
                              #:guile guile
                              #:system system
                              #:hash-algo hash-algo
@@ -492,7 +542,15 @@ name in the store."
                              #:executable? executable?
                              #:mirrors %mirror-file
                              #:content-addressed-mirrors
-                             %content-addressed-mirror-file)))))
+                             (match (%download-fallback-test)
+                               ((or #f 'content-addressed-mirrors)
+                                %content-addressed-mirror-file)
+                               (_ %no-mirrors-file))
+                             #:disarchive-mirrors
+                             (match (%download-fallback-test)
+                               ((or #f 'disarchive-mirrors)
+                                %disarchive-mirror-file)
+                               (_ %no-disarchive-mirrors-file)))))))
 
 (define* (url-fetch/executable url hash-algo hash
                                #:optional name
