@@ -1,5 +1,5 @@
 ;;; GNU Guix --- Functional package management for GNU
-;;; Copyright © 2012, 2014, 2015, 2017, 2019, 2020 Ludovic Courtès <ludo@gnu.org>
+;;; Copyright © 2012, 2014, 2015, 2017, 2019, 2020, 2022 Ludovic Courtès <ludo@gnu.org>
 ;;; Copyright © 2013, 2015 Andreas Enge <andreas@enge.fr>
 ;;; Copyright © 2012, 2013 Nikita Karetnikov <nikita@karetnikov.org>
 ;;; Copyright © 2015 Mark H Weaver <mhw@netris.org>
@@ -17,6 +17,9 @@
 ;;; Copyright © 2020 André Batista <nandre@riseup.net>
 ;;; Copyright © 2020 Helio Machado <0x2b3bfa0+guix@googlemail.com>
 ;;; Copyright © 2021 Felix Gruber <felgru@posteo.net>
+;;; Copyright © 2021 Xinglu Chen <public@yoctocell.xyz>
+;;; Copyright © 2021 Noisytoot <noisytoot@disroot.org>
+;;; Copyright © 2021 Kyle Meyer <kyle@kyleam.com>
 ;;;
 ;;; This file is part of GNU Guix.
 ;;;
@@ -40,12 +43,11 @@
             apsl2
             asl1.1 asl2.0
             boost1.0
-            bsd-0 bsd-2 bsd-3 bsd-4
+            bsd-0 bsd-1 bsd-2 bsd-3 bsd-4
             non-copyleft
             cc0
             cc-by2.0 cc-by3.0 cc-by4.0
             cc-by-sa2.0 cc-by-sa3.0 cc-by-sa4.0
-            cc-sampling-plus-1.0
             cddl1.0 cddl1.1
             cecill cecill-b cecill-c
             artistic2.0 clarified-artistic
@@ -55,7 +57,9 @@
             edl1.0
             epl1.0
             epl2.0
-            expat
+            eupl1.1
+            eupl1.2
+            expat expat-0
             freetype
             freebsd-doc
             giftware
@@ -106,13 +110,6 @@
             hpnd
             fsdg-compatible))
 
-(define-record-type <license>
-  (license name uri comment)
-  license?
-  (name    license-name)
-  (uri     license-uri)
-  (comment license-comment))
-
 ;;; Commentary:
 ;;;
 ;;; Available licenses.
@@ -126,6 +123,53 @@
 ;;;
 ;;; Code:
 
+(define-record-type <license>
+  (license name uri comment)
+  actual-license?
+  (name    license-name)
+  (uri     license-uri)
+  (comment license-comment))
+
+(define-syntax define-license-predicate
+  (syntax-rules (define define*)
+    "Define PREDICATE as a license predicate that, when applied to trivial
+cases, reduces to #t at macro-expansion time."
+    ((_ predicate (variables ...) (procedures ...)
+        (define variable _) rest ...)
+     (define-license-predicate
+       predicate
+       (variable variables ...) (procedures ...)
+       rest ...))
+    ((_ predicate (variables ...) (procedures ...)
+        (define* (procedure _ ...) _ ...)
+        rest ...)
+     (define-license-predicate
+       predicate
+       (variables ...) (procedure procedures ...)
+       rest ...))
+    ((_ predicate (variables ...) (procedures ...))
+     (define-syntax predicate
+       (lambda (s)
+         (syntax-case s (variables ... procedures ...)
+           ((_ variables) #t) ...
+           ((_ (procedures _)) #t) ...
+           ((_ obj) #'(actual-license? obj))
+           (id
+            (identifier? #'id)
+            #'actual-license?)))))))
+
+(define-syntax begin-license-definitions
+  (syntax-rules ()
+    ((_ predicate definitions ...)
+     (begin
+       ;; Define PREDICATE such that it expands to #t when passed one of the
+       ;; identifiers in DEFINITIONS.
+       (define-license-predicate predicate () () definitions ...)
+
+       definitions ...))))
+
+(begin-license-definitions license?
+
 (define agpl1
   (license "AGPL 1"
            "https://gnu.org/licenses/agpl.html"
@@ -166,6 +210,11 @@
            "https://spdx.org/licenses/0BSD.html"
            "https://opensource.org/licenses/0BSD"))
 
+(define bsd-1
+  (license "BSD 1-Clause"
+           "https://spdx.org/licenses/BSD-1-Clause.html"
+           "https://opensource.org/licenses/BSD-1-Clause"))
+
 (define bsd-2
   (license "FreeBSD"
            "http://directory.fsf.org/wiki/License:FreeBSD"
@@ -227,11 +276,6 @@ at URI, which may be a file:// URI pointing the package's tree."
            "http://creativecommons.org/licenses/by/2.0/"
            "Creative Commons Attribution 2.0 Generic"))
 
-(define cc-sampling-plus-1.0
-  (license "CC-Sampling+ 1.0"
-           "https://creativecommons.org/licenses/sampling+/1.0"
-           "Creative Commons Sampling Plus 1.0"))
-
 (define cddl1.0
   (license "CDDL 1.0"
            "http://directory.fsf.org/wiki/License:CDDLv1.0"
@@ -301,11 +345,30 @@ at URI, which may be a file:// URI pointing the package's tree."
            "https://www.eclipse.org/legal/epl-2.0/"
            "https://www.gnu.org/licenses/license-list#EPL2"))
 
+(define eupl1.1
+  (license "EUPL 1.1"
+           "https://directory.fsf.org/wiki/License:EUPL-1.1"
+           "https://www.gnu.org/licenses/license-list#EUPL-1.1"))
+
+(define eupl1.2
+  (license "EUPL 1.2"
+           "https://directory.fsf.org/wiki/License:EUPL-1.2"
+           "https://www.gnu.org/licenses/license-list#EUPL-1.2"))
+
+;; Some people call it the MIT license. For clarification see:
+;; https://www.gnu.org/licenses/license-list.html#Expat
 (define expat
   (license "Expat"
            "http://directory.fsf.org/wiki/License:Expat"
            "https://www.gnu.org/licenses/license-list.html#Expat"))
 
+(define expat-0
+  (license "Expat No Attribution"
+           ;; Note: There is a later formulation of the same license at
+           ;; <https://github.com/aws/mit-0>.
+           "https://romanrm.net/mit-zero"
+           "Expat license with the attribution paragraph removed."))
+
 (define freetype
   (license "Freetype"
            "http://directory.fsf.org/wiki/License:Freetype"
@@ -700,6 +763,6 @@ Data.  More details can be found at URI.  See also
 https://www.gnu.org/distros/free-system-distribution-guidelines.en.html#non-functional-data."
   (license "FSDG-compatible"
            uri
-           comment))
+           comment)))
 
 ;;; licenses.scm ends here
