@@ -1,5 +1,5 @@
 ;;; GNU Guix --- Functional package management for GNU
-;;; Copyright © 2017, 2018, 2019, 2020 Ludovic Courtès <ludo@gnu.org>
+;;; Copyright © 2017-2022 Ludovic Courtès <ludo@gnu.org>
 ;;; Copyright © 2018, 2019 Ricardo Wurmus <rekado@elephly.net>
 ;;;
 ;;; This file is part of GNU Guix.
@@ -23,8 +23,7 @@
   #:use-module (guix colors)
   #:use-module (guix progress)
   #:autoload   (guix build syscalls) (terminal-columns)
-  #:use-module ((guix build download)
-                #:select (nar-uri-abbreviation))
+  #:autoload   (guix build download) (nar-uri-abbreviation)
   #:use-module (guix store)
   #:use-module (guix derivations)
   #:use-module (guix memoization)
@@ -380,6 +379,10 @@ the current build phase."
      (G_ "building GHC package cache..."))
     ('ca-certificate-bundle
      (G_ "building CA certificate bundle..."))
+    ('emacs-subdirs
+     (G_ "listing Emacs sub-directories..."))
+    ('gdk-pixbuf-loaders-cache-file
+     (G_ "generating GdkPixbuf loaders cache..."))
     ('glib-schemas
      (G_ "generating GLib schema cache..."))
     ('gtk-icon-themes
@@ -392,8 +395,8 @@ the current build phase."
      (G_ "building XDG MIME database..."))
     ('fonts-dir
      (G_ "building fonts directory..."))
-    ('texlive-configuration
-     (G_ "building TeX Live configuration..."))
+    ('texlive-font-maps
+     (G_ "building TeX Live font maps..."))
     ('manual-database
      (G_ "building database for manual pages..."))
     ('package-cache                    ;package cache generated by 'guix pull'
@@ -404,11 +407,18 @@ the current build phase."
                             #:optional (port (current-error-port))
                             #:key
                             (colorize? (color-output? port))
+                            (print-urls? #t)
                             (print-log? #t))
   "Print information about EVENT and STATUS to PORT.  When COLORIZE? is true,
 produce colorful output.  When PRINT-LOG? is true, display the build log in
-addition to build events."
+addition to build events.  When PRINT-URLS? is true, display the URL of
+substitutes being downloaded."
   (define info
+    (if (and colorize? (or print-urls? print-log?))
+        (cute colorize-string <> (color BOLD))
+        identity))
+
+  (define emph
     (if colorize?
         (cute colorize-string <> (color BOLD))
         identity))
@@ -423,6 +433,9 @@ addition to build events."
         (cute colorize-string <> (color RED BOLD))
         identity))
 
+  (define tty?
+    (isatty?* port))
+
   (define (report-build-progress phase %)
     (let ((% (min (max % 0) 100)))                ;sanitize
       (erase-current-line port)
@@ -475,7 +488,9 @@ addition to build events."
              (format port (info (N_ "applying ~a graft for ~a ..."
                                     "applying ~a grafts for ~a ..."
                                     count))
-                     count drv)))
+                     count
+                     (string-drop-right (store-path-package-name drv)
+                                        (string-length ".drv")))))
          ('profile
           (let ((count (match (assq-ref properties 'profile)
                          (#f  0)
@@ -488,7 +503,7 @@ addition to build events."
           (let ((hook-type (assq-ref properties 'hook)))
             (or (and=> (hook-message hook-type)
                        (lambda (msg)
-                         (format port (info msg))))
+                         (display (info msg) port)))
                 (format port (info (G_ "running profile hook of type '~a'..."))
                         hook-type))))
          (_
@@ -516,7 +531,7 @@ addition to build events."
         (format port (failure (G_ "Could not find build log for '~a'."))
                 drv))
        (log
-        (format port (info (G_ "View build log at '~a'.")) log)))
+        (format port (emph (G_ "View build log at '~a'.")) log)))
      (newline port))
     (('substituter-started item _ ...)
      (erase-current-line*)
@@ -524,9 +539,10 @@ addition to build events."
        (format port (info (G_ "substituting ~a...")) item)
        (newline port)))
     (('download-started item uri _ ...)
-     (erase-current-line*)
-     (format port (info (G_ "downloading from ~a ...")) uri)
-     (newline port))
+     (when print-urls?
+       (erase-current-line*)
+       (format port (info (G_ "downloading from ~a ...")) uri)
+       (newline port)))
     (('download-progress item uri
                          (= string->number size)
                          (= string->number transferred))
@@ -542,16 +558,22 @@ addition to build events."
                          (nar-uri-abbreviation uri)
                          (basename uri))))
             (display-download-progress uri size
+                                       #:tty? tty?
                                        #:start-time
                                        (download-start download)
                                        #:transferred transferred))))))
     (('substituter-succeeded item _ ...)
-     ;; If there are no jobs running, we already reported download completion
-     ;; so there's nothing left to do.
-     (unless (and (zero? (simultaneous-jobs status))
-                  (extended-build-trace-supported?))
-       (format port (success (G_ "substitution of ~a complete")) item)
-       (newline port)))
+     (when (extended-build-trace-supported?)
+       ;; If there are no jobs running, we already reported download completion
+       ;; so there's nothing left to do.
+       (unless (zero? (simultaneous-jobs status))
+         (format port (success (G_ "substitution of ~a complete")) item)
+         (newline port))
+
+       (when (and print-urls? (zero? (simultaneous-jobs status)))
+         ;; Leave a blank line after the "downloading ..." line and the
+         ;; progress bar (that's three lines in total).
+         (newline port))))
     (('substituter-failed item _ ...)
      (format port (failure (G_ "substitution of ~a failed")) item)
      (newline port))
@@ -560,12 +582,12 @@ addition to build events."
      ;; /gnu/store/…-sth:", where "sha256" is the hash algorithm.
      (format port (failure (G_ "~a hash mismatch for ~a:")) algo item)
      (newline port)
-     (format port (info (G_ "\
+     (format port (emph (G_ "\
   expected hash: ~a
   actual hash:   ~a~%"))
              expected actual))
     (('build-remote drv host _ ...)
-     (format port (info (G_ "offloading build of ~a to '~a'")) drv host)
+     (format port (emph (G_ "offloading build of ~a to '~a'")) drv host)
      (newline port))
     (('build-log pid line)
      (if (multiplexed-output-supported?)
@@ -599,6 +621,17 @@ addition to build events."
                                   (colorize? (color-output? port)))
   (print-build-event event old-status status port
                      #:colorize? colorize?
+                     #:print-urls? #f
+                     #:print-log? #f))
+
+(define* (print-build-event/quiet-with-urls event old-status status
+                                            #:optional
+                                            (port (current-error-port))
+                                            #:key
+                                            (colorize? (color-output? port)))
+  (print-build-event event old-status status port
+                     #:colorize? colorize?
+                     #:print-urls? #t             ;show download URLs
                      #:print-log? #f))
 
 (define* (build-status-updater #:optional (on-change (const #t)))
@@ -634,13 +667,14 @@ case where BV does not contain only valid UTF-8."
           (close-port port)
           str)))))
 
-(define (bytevector-index bv number offset count)
-  "Search for NUMBER in BV starting from OFFSET and reading up to COUNT bytes;
-return the offset where NUMBER first occurs or #f if it could not be found."
+(define (bytevector-index bv numbers offset count)
+  "Search for NUMBERS in BV starting from OFFSET and reading up to COUNT bytes;
+return the offset where one of NUMBERS first occurs or #f if they could not be
+found."
   (let loop ((offset offset)
              (count count))
     (cond ((zero? count) #f)
-          ((= (bytevector-u8-ref bv offset) number) offset)
+          ((memv (bytevector-u8-ref bv offset) numbers) offset)
           (else (loop (+ 1 offset) (- count 1))))))
 
 (define (split-lines str)
@@ -741,7 +775,12 @@ The second return value is a thunk to retrieve the current state."
             (set! %build-output '())
             (set! %build-output-pid #f))
           keep)
-        (match (bytevector-index bv (char->integer #\newline)
+
+        ;; Search for both '\n' and '\r'; the latter is appears in progress
+        ;; messages sent by 'guix substitute' through the daemon.
+        (match (bytevector-index bv
+                                 (list (char->integer #\newline)
+                                       (char->integer #\return))
                                  offset count)
           ((? integer? cr)
            (let* ((tail (maybe-utf8->string
@@ -784,6 +823,7 @@ evaluate EXP... in that context."
   "Return the logging procedure that corresponds to LEVEL."
   (cond ((<= level 0) (const #t))
         ((= level 1)  print-build-event/quiet)
+        ((= level 2)  print-build-event/quiet-with-urls)
         (else         print-build-event)))
 
 (define (call-with-status-verbosity level thunk)
