@@ -1,5 +1,5 @@
 # GNU Guix --- Functional package management for GNU
-# Copyright © 2014, 2015, 2016, 2017, 2018, 2019, 2020, 2021 Ludovic Courtès <ludo@gnu.org>
+# Copyright © 2014-2022 Ludovic Courtès <ludo@gnu.org>
 # Copyright © 2017 Tobias Geerinckx-Rice <me@tobias.gr>
 # Copyright © 2018 Chris Marusich <cmmarusich@gmail.com>
 #
@@ -115,7 +115,7 @@ cat > "$tmpfile" <<EOF
   (timezone "Europe/Paris")                           ; 6
   (locale "en_US.UTF-8")                              ; 7
 
-  (bootloader (GRUB-config (target "/dev/sdX")))      ; 9
+  (bootloader (GRUB-config (targets (list "/dev/sdX"))))        ; 9
   (file-systems (cons (file-system
                         (device (file-system-label "root"))
                         (mount-point "/")
@@ -168,7 +168,7 @@ OS_BASE='
 
   (bootloader (bootloader-configuration
                (bootloader grub-bootloader)
-               (target "/dev/sdX")))
+               (targets (list "/dev/sdX"))))
   (file-systems (cons (file-system
                         (device (file-system-label "root"))
                         (mount-point "/")
@@ -241,7 +241,7 @@ make_user_config ()
 
   (bootloader (bootloader-configuration
                 (bootloader grub-bootloader)
-                (target "/dev/sdX")))
+                (targets (list "/dev/sdX"))))
   (file-systems (cons (file-system
                         (device (file-system-label "root"))
                         (mount-point "/")
@@ -335,18 +335,35 @@ rm "$tmpdir/search"
 
 # Verify that the examples can be built.
 for example in gnu/system/examples/*.tmpl; do
-    if echo "$example" | grep hurd; then
-        target="--target=i586-pc-gnu"
-    else
-        target=
-    fi
-    guix system -n disk-image $target "$example"
+    case "$example" in
+	*hurd*)
+            options="--target=i586-pc-gnu";;
+	*asus*)
+	    # 'asus-c201.tmpl' uses 'linux-libre-arm-generic', which is an
+	    # ARM-only package.
+            options="--system=armhf-linux";;
+	*vm-image*)
+	    # The VM image tries to build 'current-guix' as per 'guix pull'.
+	    # Skip it.
+	    continue
+	    ;;
+	*)
+	    options=""
+	    ;;
+    esac
+    guix system -n disk-image $options "$example"
+done
+
+# Make sure the desktop image can be built on major architectures.
+for system in x86_64-linux i686-linux aarch64-linux
+do
+    guix system -n image -s "$system" gnu/system/examples/desktop.tmpl
 done
 
 # Verify that the images can be built.
-guix system -n vm gnu/system/examples/vm-image.tmpl
+guix system -n vm gnu/system/examples/bare-bones.tmpl
 guix system -n image gnu/system/images/pinebook-pro.scm
-guix system -n image -t qcow2 gnu/system/examples/vm-image.tmpl
+guix system -n image -t qcow2 gnu/system/examples/bare-bones.tmpl
 guix system -n image -t iso9660 gnu/system/examples/bare-bones.tmpl
 guix system -n docker-image gnu/system/examples/docker-image.tmpl
 
