@@ -1,5 +1,5 @@
 ;;; GNU Guix --- Functional package management for GNU
-;;; Copyright © 2016, 2018 Ludovic Courtès <ludo@gnu.org>
+;;; Copyright © 2016, 2018, 2022 Ludovic Courtès <ludo@gnu.org>
 ;;; Copyright © 2017 Tobias Geerinckx-Rice <me@tobias.gr>
 ;;;
 ;;; This file is part of GNU Guix.
@@ -21,6 +21,9 @@
   #:use-module (gnu)
   #:use-module ((gnu services) #:select (service-value))
   #:use-module (guix store)
+  #:use-module (guix monads)
+  #:use-module ((guix gexp) #:select (lower-object))
+  #:use-module ((guix utils) #:select (%current-system))
   #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-64))
 
@@ -39,7 +42,7 @@
     (locale "en_US.utf8")
     (bootloader (bootloader-configuration
                  (bootloader grub-bootloader)
-                 (target "/dev/sdX")))
+                 (targets '("/dev/sdX"))))
     (file-systems (cons %root-fs %base-file-systems))
 
     (users %base-user-accounts)))
@@ -56,7 +59,7 @@
     (locale "en_US.utf8")
     (bootloader (bootloader-configuration
                  (bootloader grub-bootloader)
-                 (target "/dev/sdX")))
+                 (targets '("/dev/sdX"))))
     (mapped-devices (list %luks-device))
     (file-systems (cons (file-system
                           (inherit %root-fs)
@@ -64,6 +67,8 @@
                         %base-file-systems))
     (users %base-user-accounts)))
 
+(%graft? #f)
+
 
 (test-begin "system")
 
@@ -140,4 +145,17 @@
                            (type "ext4")
                            (dependencies (list %luks-device))))))))))
 
+(test-assert "lower-object, %current-system sensitivity"
+  ;; Make sure that 'lower-object' returns the same derivation, no matter what
+  ;; '%current-system' is.  See <https://issues.guix.gnu.org/55951>.
+  (let ((drv1 (with-store store
+                (parameterize ((%current-system "x86_64-linux"))
+                  (run-with-store store
+                    (lower-object %os "aarch64-linux")))))
+        (drv2 (with-store store
+                (parameterize ((%current-system "aarch64-linux"))
+                  (run-with-store store
+                    (lower-object %os "aarch64-linux"))))))
+    (eq? drv1 drv2)))
+
 (test-end)
